@@ -1,10 +1,10 @@
 /*
  *
  * $Id.cpp,v 1.82 2005/02/04 09:27:19 trueg Exp $
- * Copyright (C) 2003-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <k3bcddbquery.h>
 #include <k3bcore.h>
 #include <k3binffilewriter.h>
+#include <k3bglobalsettings.h>
 
 #include <kconfig.h>
 #include <kstandarddirs.h>
@@ -41,7 +42,6 @@
 #include <kio/netaccess.h>
 #include <kio/job.h>
 #include <kio/global.h>
-#include <kmessagebox.h>
 
 #include <qtimer.h>
 #include <qstringlist.h>
@@ -122,9 +122,11 @@ K3bCdCopyJob::K3bCdCopyJob( K3bJobHandler* hdl, QObject* parent )
     m_copies(1),
     m_onlyCreateImages(false),
     m_onTheFly(true),
-    m_ignoreReadErrors(false),
+    m_ignoreDataReadErrors(false),
+    m_ignoreAudioReadErrors(true),
     m_noCorrection(false),
-    m_readRetries(128),
+    m_dataReadRetries(128),
+    m_audioReadRetries(5),
     m_preferCdText(false),
     m_copyCdText(true),
     m_writingMode( K3b::WRITING_MODE_AUTO )
@@ -150,15 +152,15 @@ void K3bCdCopyJob::start()
   d->sessionSizes.clear();
   d->dataSessionProbablyTAORecorded.clear();
   d->deleteTempDir = false;
+  d->haveCdText = false;
+  d->haveCddb = false;
 
-  emit started();
+  jobStarted();
 
-  emit newTask( i18n("Checking Source Disk") );
-
-  // without this message the window looks quite empty and the user might think nothing is going on
-  emit infoMessage( i18n("Checking source disk"), INFO );
+  emit newTask( i18n("Checking Source Medium") );
 
   emit burning(false);
+  emit newSubTask( i18n("Waiting for source medium") );
 
   // wait for a source disk
   if( waitForMedia( m_readerDevice,
@@ -168,7 +170,7 @@ void K3bCdCopyJob::start()
     return;
   }
 
-  emit newSubTask( i18n("Reading Table of Contents") );
+  emit newSubTask( i18n("Checking source medium") );
 
   // FIXME: read ISRCs and MCN
 
@@ -184,7 +186,7 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 
     //
     // for now we copy audio, pure data (aka 1 data track), cd-extra (2 session, audio and data),
-    // and data multisession wich one track per session.
+    // and data multisession which one track per session.
     // Everything else will be rejected
     //
     bool canCopy = true;
@@ -204,7 +206,7 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
       else
 	emit infoMessage( i18n("Copying Data CD."), INFO );
       break;
-      
+
     case K3bDevice::MIXED:
       audio = true;
       if( dh->diskInfo().numSessions() != 2 || d->toc[0].type() != K3bDevice::Track::AUDIO ) {
@@ -258,7 +260,7 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
     //
     // To copy mode2 data tracks we need cdrecord >= 2.01a12 which introduced the -xa1 and -xamix options
     //
-    if( k3bcore->externalBinManager()->binObject("cdrecord") && 
+    if( k3bcore->externalBinManager()->binObject("cdrecord") &&
 	!k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "xamix" ) ) {
       for( K3bDevice::Toc::const_iterator it = d->toc.begin(); it != d->toc.end(); ++it ) {
 	if( (*it).type() == K3bDevice::Track::DATA &&
@@ -326,6 +328,11 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
     }
 
     if( canCopy ) {
+      if( K3b::isMounted( m_readerDevice ) ) {
+	emit infoMessage( i18n("Unmounting source medium"), INFO );
+	K3b::unmount( m_readerDevice );
+      }
+
       d->overallSize = 0;
 
       // now create some progress helper values
@@ -360,10 +367,10 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 void K3bCdCopyJob::searchCdText()
 {
   emit newSubTask( i18n("Searching CD-TEXT") );
-  
+
   connect( K3bDevice::sendCommand( K3bDevice::DeviceHandler::CD_TEXT_RAW, m_readerDevice ),
 	   SIGNAL(finished(K3bDevice::DeviceHandler*)),
-	   this, 
+	   this,
 	   SLOT(slotCdTextReady(K3bDevice::DeviceHandler*)) );
 }
 
@@ -385,7 +392,7 @@ void K3bCdCopyJob::slotCdTextReady( K3bDevice::DeviceHandler* dh )
     if( d->haveCdText && m_preferCdText )
       startCopy();
     else
-      queryCddb();  
+      queryCddb();
   }
   else {
     emit infoMessage( i18n("No CD-TEXT found."), INFO );
@@ -446,7 +453,7 @@ void K3bCdCopyJob::startCopy()
 {
   d->currentWrittenSession = d->currentReadSession = 1;
   d->doneCopies = 0;
-  
+
   if( m_onTheFly ) {
     emit newSubTask( i18n("Preparing write process...") );
 
@@ -547,11 +554,9 @@ bool K3bCdCopyJob::prepareImageFiles()
   }
   else {
     // we only need a single image file
-    if( !fi.isFile() || 
-	KMessageBox::warningYesNo( qApp->activeWindow(),
-				   i18n("Do you want to overwrite %1?").arg(m_tempPath),
-				   i18n("File Exists") )
-	== KMessageBox::Yes) {
+    if( !fi.isFile() ||
+	questionYesNo( i18n("Do you want to overwrite %1?").arg(m_tempPath),
+		       i18n("File Exists") ) ) {
       if( fi.isDir() )
 	m_tempPath = K3b::findTempFile( "iso", m_tempPath );
       else if( !QFileInfo( m_tempPath.section( '/', 0, -2 ) ).isDir() ) {
@@ -578,7 +583,7 @@ void K3bCdCopyJob::readNextSession()
     if( d->numSessions > 1 )
       emit newTask( i18n("Reading Session %1").arg(d->currentReadSession) );
     else
-      emit newTask( i18n("Reading Source Disk") );
+      emit newTask( i18n("Reading Source Medium") );
 
     if( d->currentReadSession == 1 )
       emit newSubTask( i18n("Reading track %1 of %2").arg(1).arg(d->toc.count()) );
@@ -590,9 +595,9 @@ void K3bCdCopyJob::readNextSession()
   if( d->currentReadSession == 1 && d->toc[0].type() == K3bDevice::Track::AUDIO ) {
     if( !d->audioSessionReader ) {
       d->audioSessionReader = new K3bAudioSessionReadingJob( this, this );
-      connect( d->audioSessionReader, SIGNAL(nextTrack(int, int)), 
+      connect( d->audioSessionReader, SIGNAL(nextTrack(int, int)),
 	       this, SLOT(slotReadingNextTrack(int, int)) );
-      connectSubJob( d->audioSessionReader, 
+      connectSubJob( d->audioSessionReader,
 		     SLOT(slotSessionReaderFinished(bool)),
 		     true,
 		     SLOT(slotReaderProgress(int)),
@@ -602,8 +607,8 @@ void K3bCdCopyJob::readNextSession()
     d->audioSessionReader->setDevice( m_readerDevice );
     d->audioSessionReader->setToc( d->toc );
     d->audioSessionReader->setParanoiaMode( m_paranoiaMode );
-    d->audioSessionReader->setReadRetries( m_readRetries );
-    d->audioSessionReader->setNeverSkip( !m_ignoreReadErrors );
+    d->audioSessionReader->setReadRetries( m_audioReadRetries );
+    d->audioSessionReader->setNeverSkip( !m_ignoreAudioReadErrors );
     if( m_onTheFly )
       d->audioSessionReader->writeToFd( d->cdrecordWriter->fd() );
     else
@@ -619,14 +624,14 @@ void K3bCdCopyJob::readNextSession()
       connect( d->dataTrackReader, SIGNAL(processedSize(int, int)), this, SLOT(slotReaderProcessedSize(int, int)) );
       connect( d->dataTrackReader, SIGNAL(finished(bool)), this, SLOT(slotSessionReaderFinished(bool)) );
       connect( d->dataTrackReader, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
-      connect( d->dataTrackReader, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+      connect( d->dataTrackReader, SIGNAL(debuggingOutput(const QString&, const QString&)),
 	       this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
     }
 
     d->dataTrackReader->setDevice( m_readerDevice );
-    d->dataTrackReader->setIgnoreErrors( m_ignoreReadErrors );
+    d->dataTrackReader->setIgnoreErrors( m_ignoreDataReadErrors );
     d->dataTrackReader->setNoCorrection( m_noCorrection );
-    d->dataTrackReader->setRetries( m_readRetries );
+    d->dataTrackReader->setRetries( m_dataReadRetries );
     if( m_onlyCreateImages )
       d->dataTrackReader->setSectorSize( K3bDataTrackReader::MODE1 );
     else
@@ -658,10 +663,11 @@ void K3bCdCopyJob::readNextSession()
       d->dataTrackReader->writeToFd( d->cdrecordWriter->fd() );
     else
       d->dataTrackReader->setImagePath( d->imageNames[trackNum-1] );
-    
+
     d->dataReaderRunning = true;
     if( !m_onTheFly || m_onlyCreateImages )
       slotReadingNextTrack( 1, 1 );
+
     d->dataTrackReader->start();
   }
 }
@@ -690,7 +696,7 @@ bool K3bCdCopyJob::writeNextSession()
   emit newSubTask( i18n("Waiting for media") );
 
   // if session > 1 we wait for an appendable CD
-  if( waitForMedia( m_writerDevice, 
+  if( waitForMedia( m_writerDevice,
 		    d->currentWrittenSession > 1 && !m_simulate
 		    ? K3bDevice::STATE_INCOMPLETE
 		    : K3bDevice::STATE_EMPTY,
@@ -714,7 +720,7 @@ bool K3bCdCopyJob::writeNextSession()
     connect( d->cdrecordWriter, SIGNAL(finished(bool)), this, SLOT(slotWriterFinished(bool)) );
     //    connect( d->cdrecordWriter, SIGNAL(newTask(const QString&)), this, SIGNAL(newTask(const QString&)) );
     connect( d->cdrecordWriter, SIGNAL(newSubTask(const QString&)), this, SIGNAL(newSubTask(const QString&)) );
-    connect( d->cdrecordWriter, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+    connect( d->cdrecordWriter, SIGNAL(debuggingOutput(const QString&, const QString&)),
 	     this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
   }
 
@@ -722,6 +728,7 @@ bool K3bCdCopyJob::writeNextSession()
   d->cdrecordWriter->clearArguments();
   d->cdrecordWriter->setSimulate( m_simulate );
   d->cdrecordWriter->setBurnSpeed( m_speed );
+  d->cdrecordWriter->setForceNoEject( true );
 
 
   // create the cdrecord arguments
@@ -874,10 +881,11 @@ bool K3bCdCopyJob::writeNextSession()
     bool multi = d->doNotCloseLastSession || (d->numSessions > 1 && d->currentWrittenSession < d->toc.count());
     int usedWritingMode = m_writingMode;
     if( usedWritingMode == K3b::WRITING_MODE_AUTO ) {
-      // use DAO in overwriting mode when not writing multi-session or multi-track
-      k3bcore->config()->setGroup("General Options");
+      // at least the NEC3540a does write 2056 byte sectors only in tao mode. Same for LG4040b
+      // since writing data tracks in TAO mode is no loss let's default to TAO in the case of 2056 byte
+      // sectors (which is when writing xa form1 sectors here)
       if( m_writerDevice->dao() &&
-	  d->toc.count() == 1 && 
+	  d->toc.count() == 1 &&
 	  !multi &&
 	  track->mode() == K3bDevice::Track::MODE1 )
  	usedWritingMode = K3b::DAO;
@@ -964,12 +972,10 @@ void K3bCdCopyJob::slotSessionReaderFinished( bool success )
 	    // becasue if it did not it might happen that k3b overwrites a CD-RW
 	    // source)
 	    if( !m_readerDevice->eject() ) {
-	      emit infoMessage( i18n("Unable to eject media."), ERROR );
-	      finishJob( false, true );
-	      return;
+	      blockingInformation( i18n("K3b was unable to eject the source disk. Please do so manually.") );
 	    }
 	  }
-	  
+
 	  if( !writeNextSession() ) {
 	    // nothing is running here...
 	    finishJob( d->canceled, d->error );
@@ -985,7 +991,7 @@ void K3bCdCopyJob::slotSessionReaderFinished( bool success )
     if( !d->canceled ) {
       emit infoMessage( i18n("Error while reading session %1.").arg(d->currentReadSession), ERROR );
       if( m_onTheFly )
-	d->cdrecordWriter->setSourceUnreadable(true);	
+	d->cdrecordWriter->setSourceUnreadable(true);
     }
 
     finishJob( d->canceled, !d->canceled );
@@ -1009,17 +1015,21 @@ void K3bCdCopyJob::slotWriterFinished( bool success )
       d->currentWrittenSession++;
       d->currentReadSession++;
 
-      // reload the media
-      emit newSubTask( i18n("Reloading the media") );
-      connect( K3bDevice::reload( m_writerDevice ), SIGNAL(finished(K3bDevice::DeviceHandler*)),
-	       this, SLOT(slotMediaReloadedForNextSession(K3bDevice::DeviceHandler*)) );
+      if( !writeNextSession() ) {
+          // nothing is running here...
+          finishJob( d->canceled, d->error );
+      }
+      else if( m_onTheFly )
+          readNextSession();
     }
     else {
       d->doneCopies++;
 
       if( !m_simulate && d->doneCopies < m_copies ) {
 	// start next copy
-	K3bDevice::eject( m_writerDevice );
+	if ( !m_writerDevice->eject() ) {
+            blockingInformation( i18n("K3b was unable to eject the written disk. Please do so manually.") );
+        }
 
 	d->currentWrittenSession = 1;
 	d->currentReadSession = 1;
@@ -1028,12 +1038,17 @@ void K3bCdCopyJob::slotWriterFinished( bool success )
 	    readNextSession();
 	}
 	else {
-	  // nothing running here...
-	  finishJob( d->canceled, d->error );
+            if( k3bcore->globalSettings()->ejectMedia() )
+                K3bDevice::eject( m_writerDevice );
+
+            // nothing running here...
+            finishJob( d->canceled, d->error );
 	}
       }
       else {
-	finishJob( false, false );
+          if( k3bcore->globalSettings()->ejectMedia() )
+              K3bDevice::eject( m_writerDevice );
+          finishJob( false, false );
       }
     }
   }
@@ -1050,21 +1065,6 @@ void K3bCdCopyJob::slotWriterFinished( bool success )
 }
 
 
-void K3bCdCopyJob::slotMediaReloadedForNextSession( K3bDevice::DeviceHandler* dh )
-{
-  if( !dh->success() )
-    KMessageBox::information( qApp->activeWindow(), i18n("Please reload the medium and press 'ok'"),
-			      i18n("Unable to close the tray") );
-
-  if( !writeNextSession() ) {
-    // nothing is running here...
-    finishJob( d->canceled, d->error );
-  }
-  else if( m_onTheFly )
-    readNextSession();
-}
-
-
 void K3bCdCopyJob::cleanup()
 {
   if( m_onTheFly || !m_keepImage || ((d->canceled || d->error) && !d->readingSuccessful) ) {
@@ -1080,7 +1080,7 @@ void K3bCdCopyJob::cleanup()
 
     // remove the tempdir created in prepareImageFiles()
     if( d->deleteTempDir ) {
-      KIO::NetAccess::del( KURL::fromPathOrURL(m_tempPath) );
+      KIO::NetAccess::del( KURL::fromPathOrURL(m_tempPath), 0 );
       d->deleteTempDir = false;
     }
   }
@@ -1182,8 +1182,8 @@ QString K3bCdCopyJob::jobDescription() const
 
 QString K3bCdCopyJob::jobDetails() const
 {
-  return i18n("Creating 1 copy", 
-	      "Creating %n copies", 
+  return i18n("Creating 1 copy",
+	      "Creating %n copies",
 	      (m_simulate||m_onlyCreateImages) ? 1 : m_copies );
 }
 
@@ -1197,12 +1197,12 @@ void K3bCdCopyJob::finishJob( bool c, bool e )
     }
     if( e )
       d->error = true;
-    
+
     cleanup();
-    
+
     d->running = false;
-    
-    emit finished( !(c||e) );
+
+    jobFinished( !(c||e) );
   }
 }
 
