@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3baudiojob.cpp 459254 2005-09-10 13:26:05Z trueg $
+ * $Id: k3baudiojob.cpp 630823 2007-02-06 14:07:10Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "k3baudiojobtempdata.h"
 #include "k3baudiomaxspeedjob.h"
 #include "k3baudiocdtracksource.h"
+#include "k3baudiofile.h"
 #include <k3bdevicemanager.h>
 #include <k3bdevicehandler.h>
 #include <k3bdevice.h>
@@ -43,6 +44,32 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <ktempfile.h>
+#include <kstringhandler.h>
+
+
+
+static QString createNonExistingFilesString( const QValueList<K3bAudioFile*>& items, unsigned int max )
+{
+  QString s;
+  unsigned int cnt = 0;
+  for( QValueList<K3bAudioFile*>::const_iterator it = items.begin();
+       it != items.end(); ++it ) {
+
+    s += KStringHandler::csqueeze( (*it)->filename(), 60 );
+
+    ++cnt;
+    if( cnt >= max || it == items.end() )
+      break;
+
+    s += "<br>";
+  }
+
+  if( items.count() > max )
+    s += "...";
+
+  return s;
+}
+
 
 
 class K3bAudioJob::Private
@@ -59,6 +86,9 @@ class K3bAudioJob::Private
 
   bool useCdText;
   bool maxSpeed;
+
+  bool zeroPregap;
+  bool less4Sec;
 };
 
 
@@ -95,7 +125,10 @@ K3bAudioJob::~K3bAudioJob()
 
 K3bDevice::Device* K3bAudioJob::writer() const
 {
-  return m_doc->burner();
+  if( m_doc->onlyCreateImages() )
+    return 0; // no writer needed -> no blocking on K3bBurnJob
+  else
+    return m_doc->burner();
 }
 
 
@@ -107,7 +140,7 @@ K3bDoc* K3bAudioJob::doc() const
 
 void K3bAudioJob::start()
 {
-  emit started();
+  jobStarted();
 
   m_written = true;
   m_canceled = false;
@@ -123,51 +156,102 @@ void K3bAudioJob::start()
 
   emit newTask( i18n("Preparing data") );
 
+  //
+  // Check if all files exist
+  //
+  QValueList<K3bAudioFile*> nonExistingFiles;
+  K3bAudioTrack* track = m_doc->firstTrack();
+  while( track ) {
+    K3bAudioDataSource* source = track->firstSource();
+    while( source ) {
+      if( K3bAudioFile* file = dynamic_cast<K3bAudioFile*>( source ) ) {
+	if( !QFile::exists( file->filename() ) )
+	  nonExistingFiles.append( file );
+      }
+      source = source->next();
+    }
+    track = track->next();
+  }
+  if( !nonExistingFiles.isEmpty() ) {
+    if( questionYesNo( "<p>" + i18n("The following files could not be found. Do you want to remove them from the "
+				    "project and continue without adding them to the image?") + 
+		       "<p>" + createNonExistingFilesString( nonExistingFiles, 10 ),
+		       i18n("Warning"),
+		       i18n("Remove missing files and continue"),
+		       i18n("Cancel and go back") ) ) {
+      for( QValueList<K3bAudioFile*>::const_iterator it = nonExistingFiles.begin();
+	   it != nonExistingFiles.end(); ++it ) {
+	delete *it;
+      }
+    }
+    else {
+      m_canceled = true;
+      emit canceled();
+      jobFinished(false);
+      return;
+    }
+  }
+
+  //
+  // Make sure the project is not empty
+  //
+  if( m_doc->numOfTracks() == 0 ) {
+    emit infoMessage( i18n("Please add files to your project first."), ERROR );
+    jobFinished(false);
+    return;
+  }
+
   if( m_doc->onTheFly() && !checkAudioSources() ) {
     emit infoMessage( i18n("Unable to write on-the-fly with these audio sources."), WARNING );
     m_doc->setOnTheFly(false);
   }
 
+
   // we don't need this when only creating image and it is possible
   // that the burn device is null
   if( !m_doc->onlyCreateImages() ) {
+
+    //
+    // there are a lot of writers out there which produce coasters
+    // in dao mode if the CD contains pregaps of length 0 (or maybe already != 2 secs?)
+    //
+    // Also most writers do not accept cuesheets with tracks smaller than 4 seconds (a violation
+    // of the red book standard) in DAO mode.
+    //
+    d->zeroPregap = false;
+    d->less4Sec = false;
+    track = m_doc->firstTrack();
+    while( track ) {
+      if( track->postGap() == 0 && track->next() != 0 ) // the last track's postgap is always 0
+	d->zeroPregap = true;
+
+      if( track->length() < K3b::Msf( 0, 4, 0 ) )
+	d->less4Sec = true;
+
+      track = track->next();
+    }
+
     // determine writing mode
     if( m_doc->writingMode() == K3b::WRITING_MODE_AUTO ) {
       //
       // DAO is always the first choice
-      // choose TAO if the user wants to use cdrecord since
+      // RAW second and TAO last
       // there are none-DAO writers that are supported by cdrdao
       //
+      // older cdrecord versions do not support the -shorttrack option in RAW writing mode
+      //
       if( !writer()->dao() && writingApp() == K3b::CDRECORD ) {
-	if( !writer()->supportsRawWriting() )
-	  m_usedWritingMode = K3b::TAO;
-	else
+	if(!writer()->supportsRawWriting() &&
+	   ( !d->less4Sec || k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "short-track-raw" ) ) )
 	  m_usedWritingMode = K3b::RAW;
+	else
+	  m_usedWritingMode = K3b::TAO;
       }
       else {
-	//
-        // there are a lot of writers out there which produce coasters
-        // in dao mode if the CD contains pregaps of length 0 (or maybe already != 2 secs?)
-	//
-	// Also most writers do not accept cuesheets with tracks smaller than 4 seconds (a violation
-	// of the red book standard) in DAO mode.
-	//
-        bool zeroPregap = false;
-	bool less4Sec = false;
-	K3bAudioTrack* track = m_doc->firstTrack();
-        while( track ) {
-          if( track->postGap() == 0 && track->next() != 0 ) // the last track's postgap is always 0
-            zeroPregap = true;
-
-	  if( track->length() < K3b::Msf( 0, 4, 0 ) )
-	    less4Sec = true;
-
-	  track = track->next();
-        }
-        if( (zeroPregap||less4Sec) && writer()->supportsRawWriting() ) {
+        if( (d->zeroPregap||d->less4Sec) && writer()->supportsRawWriting() ) {
           m_usedWritingMode = K3b::RAW;
-	  if( less4Sec )
-	    emit infoMessage( i18n("Tracklength below 4 seconds violate the Red Book standard."), WARNING );
+	  if( d->less4Sec )
+	    emit infoMessage( i18n("Tracklengths below 4 seconds violate the Red Book standard."), WARNING );
 	}
         else
           m_usedWritingMode = K3b::DAO;
@@ -179,7 +263,7 @@ void K3bAudioJob::start()
     bool cdrecordOnTheFly = false;
     bool cdrecordCdText = false;
     if( k3bcore->externalBinManager()->binObject("cdrecord") ) {
-      cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->version >= K3bVersion( 2, 1, -1, "a13" );
+      cdrecordOnTheFly = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "audio-stdin" );
       cdrecordCdText = k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "cdtext" );
     }    
 
@@ -241,7 +325,7 @@ void K3bAudioJob::start()
     else {
       if( !prepareWriter() ) {
 	cleanupAfterError();
-	emit finished(false);
+	jobFinished(false);
 	return;
       }
       
@@ -285,7 +369,7 @@ void K3bAudioJob::slotMaxSpeedJobFinished( bool success )
   // same code as above. See the commecnts there
   if( !prepareWriter() ) {
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
     return;
   }
   
@@ -310,7 +394,7 @@ void K3bAudioJob::cancel()
   emit infoMessage( i18n("Writing canceled."), K3bJob::ERROR );
   removeBufferFiles();
   emit canceled();
-  emit finished(false);
+  jobFinished(false);
 }
 
 
@@ -321,7 +405,7 @@ void K3bAudioJob::slotWriterFinished( bool success )
 
   if( !success ) {
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
     return;
   }
   else {
@@ -331,7 +415,7 @@ void K3bAudioJob::slotWriterFinished( bool success )
       if( m_doc->onTheFly() || m_doc->removeImages() )
 	removeBufferFiles();
 
-      emit finished(true);
+      jobFinished(true);
     }
     else {
       K3bDevice::eject( m_doc->burner() );
@@ -365,7 +449,7 @@ void K3bAudioJob::slotAudioDecoderFinished( bool success )
 
     emit infoMessage( i18n("Error while decoding audio tracks."), ERROR );
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
     return;
   }
 
@@ -379,13 +463,13 @@ void K3bAudioJob::slotAudioDecoderFinished( bool success )
     else if( !m_doc->onlyCreateImages() ) {
       if( !prepareWriter() ) {
 	cleanupAfterError();
-	emit finished(false);
+	jobFinished(false);
       }
       else
 	startWriting();
     }
     else {
-      emit finished(true);
+      jobFinished(true);
     }
   }
 }
@@ -413,7 +497,7 @@ bool K3bAudioJob::prepareWriter()
 
     if( !writeInfFiles() ) {
       kdDebug() << "(K3bAudioJob) could not write inf-files." << endl;
-      emit infoMessage( i18n("IO Error"), ERROR );
+      emit infoMessage( i18n("IO Error. Most likely no space left on harddisk."), ERROR );
 
       return false;
     }
@@ -433,12 +517,17 @@ bool K3bAudioJob::prepareWriter()
     // add all the audio tracks
     writer->addArgument( "-audio" );
 
-    // we always pad because although K3b makes sure all tracks' lenght are multible of 2352
-    // it seems that normalize sometimes corrupts these lengths
-//    writer->addArgument( "-pad" );
-
-    // Allow tracks shorter than 4 seconds
-    writer->addArgument( "-shorttrack" );
+    // we only need to pad in one case. cdrecord < 2.01.01a03 cannot handle shorttrack + raw
+    if( d->less4Sec ) {
+      if( m_usedWritingMode == K3b::RAW &&
+	  !k3bcore->externalBinManager()->binObject( "cdrecord" )->hasFeature( "short-track-raw" ) ) {
+	writer->addArgument( "-pad" );
+      }
+      else {
+	// Allow tracks shorter than 4 seconds
+	writer->addArgument( "-shorttrack" );
+      }
+    }
 
     K3bAudioTrack* track = m_doc->firstTrack();
     while( track ) {
@@ -577,7 +666,7 @@ bool K3bAudioJob::startWriting()
     return false;
 
   // in case we determined the max possible writing speed we have to reset the speed on the writer job
-  // here since an inserted media is neccessary
+  // here since an inserted media is necessary
   // the Max speed job will compare the max speed value with the supported values of the writer
   if( d->maxSpeed )
     m_writer->setBurnSpeed( m_maxSpeedJob->maxSpeed() );
@@ -646,13 +735,13 @@ void K3bAudioJob::slotNormalizeJobFinished( bool success )
 
   if( success ) {
     if( m_doc->onlyCreateImages() ) {
-      emit finished(true);
+      jobFinished(true);
     }
     else {
       // start the writing
       if( !prepareWriter() ) {
 	cleanupAfterError();
-	emit finished(false);
+	jobFinished(false);
       }
       else
 	startWriting();
@@ -660,7 +749,7 @@ void K3bAudioJob::slotNormalizeJobFinished( bool success )
   }
   else {
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
   }
 }
 
