@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bdatajob.cpp 457135 2005-09-04 20:33:59Z trueg $
+ * $Id: k3bdatajob.cpp 679276 2007-06-23 13:25:21Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 #include "k3bdatadoc.h"
 #include "k3bisoimager.h"
 #include "k3bmsinfofetcher.h"
-#include "k3bdataverifyingjob.h"
 
 #include <k3bcore.h>
 #include <k3bglobals.h>
@@ -32,6 +31,9 @@
 #include <k3bcdrecordwriter.h>
 #include <k3bcdrdaowriter.h>
 #include <k3bglobalsettings.h>
+#include <k3bactivepipe.h>
+#include <k3bfilesplitter.h>
+#include <k3bverificationjob.h>
 
 #include <kprocess.h>
 #include <kapplication.h>
@@ -54,11 +56,13 @@ class K3bDataJob::Private
 {
 public:
   Private()
-    : verificationJob(0) {
+    : usedWritingApp(K3b::CDRECORD),
+      verificationJob(0) {
   }
 
   K3bDataDoc* doc;
 
+  bool initializingImager;
   bool imageFinished;
   bool canceled;
 
@@ -72,7 +76,10 @@ public:
   int copies;
   int copiesDone;
 
-  K3bDataVerifyingJob* verificationJob;
+  K3bVerificationJob* verificationJob;
+
+  K3bFileSplitter imageFile;
+  K3bActivePipe pipe;
 };
 
 
@@ -90,7 +97,7 @@ K3bDataJob::K3bDataJob( K3bDataDoc* doc, K3bJobHandler* hdl, QObject* parent )
   m_msInfoFetcher = new K3bMsInfoFetcher( this, this );
   connect( m_msInfoFetcher, SIGNAL(finished(bool)), this, SLOT(slotMsInfoFetched(bool)) );
   connect( m_msInfoFetcher, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
-  connect( m_msInfoFetcher, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+  connect( m_msInfoFetcher, SIGNAL(debuggingOutput(const QString&, const QString&)),
 	   this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
 
   d->imageFinished = true;
@@ -111,13 +118,16 @@ K3bDoc* K3bDataJob::doc() const
 
 K3bDevice::Device* K3bDataJob::writer() const
 {
-  return doc()->burner();
+  if( doc()->onlyCreateImages() )
+    return 0; // no writer needed -> no blocking on K3bBurnJob
+  else
+    return doc()->burner();
 }
 
 
 void K3bDataJob::start()
 {
-  emit started();
+  jobStarted();
 
   d->canceled = false;
   d->imageFinished = false;
@@ -134,6 +144,10 @@ void K3bDataJob::start()
 
   emit newTask( i18n("Preparing data") );
 
+  // there is no harm in setting these even if we write on-the-fly
+  d->imageFile.setName( d->doc->tempDir() );
+  d->pipe.readFromIODevice( &d->imageFile );
+
   if( d->usedMultiSessionMode == K3bDataDoc::AUTO && !d->doc->onlyCreateImages() )
     determineMultiSessionMode();
   else
@@ -143,95 +157,79 @@ void K3bDataJob::start()
 
 void K3bDataJob::prepareWriting()
 {
-  if( !d->doc->onlyCreateImages() && 
+  if( !d->doc->onlyCreateImages() &&
       ( d->usedMultiSessionMode == K3bDataDoc::CONTINUE ||
 	d->usedMultiSessionMode == K3bDataDoc::FINISH ) ) {
     // no sense continuing the same session twice
     // FIXME: why not?
     d->copies = 1;
-    
+
     m_msInfoFetcher->setDevice( d->doc->burner() );
-    waitForDisk();
-    
-    // just to be sure we did not get canceled during the async discWaiting
-    if( d->canceled )
+
+    if( !waitForMedium() ) {
+      cancel();
       return;
-    
-    if( !KIO::findDeviceMountPoint( d->doc->burner()->mountDevice() ).isEmpty() ) {
-      emit infoMessage( i18n("Unmounting disk"), INFO );
-      // unmount the cd
-      connect( KIO::unmount( d->doc->burner()->mountPoint(), false ), SIGNAL(result(KIO::Job*)),
-	       m_msInfoFetcher, SLOT(start()) );
     }
-    else
-      m_msInfoFetcher->start();
+
+    if( K3b::isMounted( d->doc->burner() ) ) {
+      emit infoMessage( i18n("Unmounting disk"), INFO );
+      K3b::unmount( d->doc->burner() );
+    }
+
+    m_msInfoFetcher->start();
   }
   else {
     m_isoImager->setMultiSessionInfo( QString::null );
-    determineWritingMode();
-    writeImage();
+    prepareData();
+
+    d->initializingImager = true;
+    m_isoImager->init();
   }
 }
 
 
 void K3bDataJob::slotMsInfoFetched(bool success)
 {
-  if( d->canceled )
-    return;
-
   if( success ) {
     // we call this here since in ms mode we might want to check
     // the last track's datamode
-    determineWritingMode();
+    prepareData();
 
-    if( d->usedWritingApp == K3b::CDRECORD )
-      m_isoImager->setMultiSessionInfo( m_msInfoFetcher->msInfo(), d->doc->burner() );
-    else  // cdrdao seems to write a 150 blocks pregap that is not used by cdrecord
+    if( d->usedWritingApp == K3b::CDRDAO )  // cdrdao seems to write a 150 blocks pregap that is not used by cdrecord
       m_isoImager->setMultiSessionInfo( QString("%1,%2").arg(m_msInfoFetcher->lastSessionStart()).arg(m_msInfoFetcher->nextSessionStart()+150), d->doc->burner() );
+    else
+      m_isoImager->setMultiSessionInfo( m_msInfoFetcher->msInfo(), d->doc->burner() );
 
-    writeImage();
+    d->initializingImager = true;
+    m_isoImager->init();
   }
   else {
     // the MsInfoFetcher already emitted failure info
     cancelAll();
+    jobFinished( false );
   }
 }
 
 
 void K3bDataJob::writeImage()
 {
-  if( d->doc->onTheFly() && !d->doc->onlyCreateImages() ) {
-    m_isoImager->calculateSize();
-  }
-  else {
-    emit burning(false);
+  d->initializingImager = false;
 
-    // get image file path
-    if( d->doc->tempDir().isEmpty() )
-      d->doc->setTempDir( K3b::findUniqueFilePrefix( d->doc->isoOptions().volumeID() ) + ".iso" );
+  emit burning(false);
 
-    // TODO: check if the image file is part of the project and if so warn the user
-    //       and append some number to make the path unique.
+  // get image file path
+  if( d->doc->tempDir().isEmpty() )
+    d->doc->setTempDir( K3b::findUniqueFilePrefix( d->doc->isoOptions().volumeID() ) + ".iso" );
 
-    emit newTask( i18n("Creating image file") );
-    emit newSubTask( i18n("Track 1 of 1") );
-    emit infoMessage( i18n("Creating image file in %1").arg(d->doc->tempDir()), INFO );
+  // TODO: check if the image file is part of the project and if so warn the user
+  //       and append some number to make the path unique.
 
-    m_isoImager->writeToImageFile( d->doc->tempDir() );
-    m_isoImager->start();
-  }
-}
+  emit newTask( i18n("Creating image file") );
+  emit newSubTask( i18n("Track 1 of 1") );
+  emit infoMessage( i18n("Creating image file in %1").arg(d->doc->tempDir()), INFO );
 
-
-void K3bDataJob::slotSizeCalculationFinished( int status, int size )
-{
-  emit debuggingOutput( "K3b", QString( "Size of filesystem calculated: %1" ).arg(size) );
-
-  //
-  // this is only called in on-the-fly mode
-  //
-  if( status == ERROR || !startOnTheFlyWriting() )
-    cancelAll();
+  m_isoImager->writeToImageFile( d->doc->tempDir() );
+  m_isoImager->start();
 }
 
 
@@ -242,6 +240,7 @@ bool K3bDataJob::startOnTheFlyWriting()
       // try a direct connection between the processes
       if( m_writerJob->fd() != -1 )
 	m_isoImager->writeToFd( m_writerJob->fd() );
+      d->initializingImager = false;
       m_isoImager->start();
       return true;
     }
@@ -255,7 +254,21 @@ void K3bDataJob::cancel()
   emit infoMessage( i18n("Writing canceled."), K3bJob::ERROR );
   emit canceled();
 
-  cancelAll();
+  if( m_writerJob && m_writerJob->active() ) {
+    //
+    // lets wait for the writer job to finish
+    // and let it finish the job for good.
+    //
+    cancelAll();
+  }
+  else {
+    //
+    // Just cancel all and return
+    // This is bad design as we should wait for all subjobs to finish
+    //
+    cancelAll();
+    jobFinished( false );
+  }
 }
 
 
@@ -284,35 +297,71 @@ void K3bDataJob::slotIsoImagerPercent( int p )
 
 void K3bDataJob::slotIsoImagerFinished( bool success )
 {
-  if( d->canceled )
-    return;
-
-  if( !d->doc->onTheFly() ||
-      d->doc->onlyCreateImages() ) {
-
+  if( d->initializingImager ) {
     if( success ) {
-      emit infoMessage( i18n("Image successfully created in %1").arg(d->doc->tempDir()), K3bJob::SUCCESS );
-      d->imageFinished = true;
-
-      if( d->doc->onlyCreateImages() ) {
-	emit finished( true );
+      if( d->doc->onTheFly() && !d->doc->onlyCreateImages() ) {
+	if( !startOnTheFlyWriting() ) {
+	  cancelAll();
+	  jobFinished( false );
+	}
       }
       else {
-	if( prepareWriterJob() )
-	  startWriterJob();
+	writeImage();
       }
     }
     else {
       if( m_isoImager->hasBeenCanceled() )
 	emit canceled();
-      else
-	emit infoMessage( i18n("Error while creating ISO image"), ERROR );
-
-      cancelAll();
+      jobFinished( false );
     }
   }
+  else {
+    // tell the writer that there won't be more data
+    if( d->doc->onTheFly() && m_writerJob )
+      m_writerJob->closeFd();
 
-  // in case we are writing on the fly we leave the error handling to slotWriterJobFinished
+    if( !d->doc->onTheFly() ||
+	d->doc->onlyCreateImages() ) {
+
+      if( success ) {
+	emit infoMessage( i18n("Image successfully created in %1").arg(d->doc->tempDir()), K3bJob::SUCCESS );
+	d->imageFinished = true;
+
+	if( d->doc->onlyCreateImages() ) {
+	  jobFinished( true );
+	}
+	else {
+	  if( prepareWriterJob() ) {
+	    startWriterJob();
+	    d->pipe.writeToFd( m_writerJob->fd(), true );
+	    d->pipe.open(true);
+	  }
+	}
+      }
+      else {
+	if( m_isoImager->hasBeenCanceled() )
+	  emit canceled();
+	else
+	  emit infoMessage( i18n("Error while creating ISO image"), ERROR );
+
+	cancelAll();
+	jobFinished( false );
+      }
+    }
+    else if( !success ) { // on-the-fly
+      //
+      // In case the imager failed let's make sure the writer does not emit an unusable
+      // error message.
+      //
+      if( m_writerJob && m_writerJob->active() )
+	m_writerJob->setSourceUnreadable( true );
+
+      // there is one special case which we need to handle here: the iso imager might be canceled
+      // FIXME: the iso imager should not be able to cancel itself
+      if( m_isoImager->hasBeenCanceled() && !this->hasBeenCanceled() )
+	cancel();
+    }
+  }
 }
 
 
@@ -328,12 +377,10 @@ bool K3bDataJob::startWriterJob()
   // if we append a new session we asked for an appendable cd already
   if( d->usedMultiSessionMode == K3bDataDoc::NONE ||
       d->usedMultiSessionMode == K3bDataDoc::START ) {
-	  
-    waitForDisk();
 
-    // just to be sure we did not get canceled during the async discWaiting
-    if( d->canceled )
+    if( !waitForMedium() ) {
       return false;
+    }
   }
 
   emit burning(true);
@@ -367,9 +414,16 @@ void K3bDataJob::slotWriterNextTrack( int t, int tt )
 
 void K3bDataJob::slotWriterJobFinished( bool success )
 {
-  if( d->canceled )
-    return;
+  d->pipe.close();
 
+  //
+  // This is a little workaround for the bad cancellation handling in this job
+  // see cancel()
+  //
+  if( d->canceled ) {
+    if( active() )
+      jobFinished( false );
+  }
 
   if( success ) {
     // allright
@@ -377,10 +431,12 @@ void K3bDataJob::slotWriterJobFinished( bool success )
 
     if( d->doc->verifyData() ) {
       if( !d->verificationJob ) {
-	d->verificationJob = new K3bDataVerifyingJob( this, this );
+	d->verificationJob = new K3bVerificationJob( this, this );
 	connect( d->verificationJob, SIGNAL(infoMessage(const QString&, int)),
 		 this, SIGNAL(infoMessage(const QString&, int)) );
-	connect( d->verificationJob, SIGNAL(newTask(const QString&)),
+ 	connect( d->verificationJob, SIGNAL(newTask(const QString&)),
+ 		 this, SIGNAL(newSubTask(const QString&)) );
+	connect( d->verificationJob, SIGNAL(newSubTask(const QString&)),
 		 this, SIGNAL(newSubTask(const QString&)) );
 	connect( d->verificationJob, SIGNAL(percent(int)),
 		 this, SLOT(slotVerificationProgress(int)) );
@@ -388,12 +444,18 @@ void K3bDataJob::slotWriterJobFinished( bool success )
 		 this, SIGNAL(subPercent(int)) );
 	connect( d->verificationJob, SIGNAL(finished(bool)),
 		 this, SLOT(slotVerificationFinished(bool)) );
+	connect( d->verificationJob, SIGNAL(debuggingOutput(const QString&, const QString&)),
+		 this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
+
       }
-      d->verificationJob->setDoc( d->doc );
+      d->verificationJob->clear();
       d->verificationJob->setDevice( d->doc->burner() );
+      d->verificationJob->setGrownSessionSize( m_isoImager->size() );
+      d->verificationJob->addTrack( 0, m_isoImager->checksum(), m_isoImager->size() );
+
+      emit burning(false);
 
       emit newTask( i18n("Verifying written data") );
-      emit burning(false);
 
       d->verificationJob->start();
     }
@@ -409,28 +471,26 @@ void K3bDataJob::slotWriterJobFinished( bool success )
 	else
 	  failed = !startWriterJob();
 
-	if( failed )
-	  cancelAll();
+	if( failed ) {
+	  cancel();
+	}
+	else if( !d->doc->onTheFly() ) {
+	  d->pipe.writeToFd( m_writerJob->fd(), true );
+	  d->pipe.open(true);
+	}
       }
       else {
-	if( !d->doc->onTheFly() && d->doc->removeImages() ) {
-	  if( QFile::exists( d->doc->tempDir() ) ) {
-	    QFile::remove( d->doc->tempDir() );
-	    emit infoMessage( i18n("Removed image file %1").arg(d->doc->tempDir()), K3bJob::SUCCESS );
-	  }
-	}
-	
-	if( d->tocFile ) {
-	  delete d->tocFile;
-	  d->tocFile = 0;
-	}
-	
-	emit finished(true);
+          if( k3bcore->globalSettings()->ejectMedia() )
+              K3bDevice::eject( d->doc->burner() );
+
+	cleanup();
+	jobFinished(true);
       }
     }
   }
   else {
     cancelAll();
+    jobFinished( false );
   }
 }
 
@@ -444,39 +504,45 @@ void K3bDataJob::slotVerificationProgress( int p )
     totalTasks+=1.0;
     tasksDone+=1.0;
   }
-  
+
   emit percent( (int)((100.0*tasksDone + (double)p) / totalTasks) );
 }
 
 
 void K3bDataJob::slotVerificationFinished( bool success )
 {
-  if( d->canceled )
-    return;
-
   d->copiesDone++;
 
-  if( !k3bcore->globalSettings()->ejectMedia() || d->copiesDone < d->copies )
+  // reconnect our imager which we deconnected for the verification
+  connectImager();
+
+  if( k3bcore->globalSettings()->ejectMedia() || d->copiesDone < d->copies )
     K3bDevice::eject( d->doc->burner() );
-  
-  if( d->copiesDone < d->copies ) {
+
+  if( !d->canceled && d->copiesDone < d->copies ) {
     bool failed = false;
     if( d->doc->onTheFly() )
       failed = !startOnTheFlyWriting();
     else
       failed = !startWriterJob();
-    
+
     if( failed )
-      cancelAll();
+      cancel();
+    else if( !d->doc->onTheFly() ) {
+      d->pipe.writeToFd( m_writerJob->fd(), true );
+      d->pipe.open(true);
+    }
+  }
+  else {
+    cleanup();
+    jobFinished( success );
   }
-  else
-    emit finished( success );
 }
 
 
 void K3bDataJob::setWriterJob( K3bAbstractWriter* writer )
 {
-  // FIXME: progressedsize for multible copies
+  // FIXME: progressedsize for multiple copies
   m_writerJob = writer;
   connect( m_writerJob, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
   connect( m_writerJob, SIGNAL(percent(int)), this, SLOT(slotWriterJobPercent(int)) );
@@ -489,19 +555,30 @@ void K3bDataJob::setWriterJob( K3bAbstractWriter* writer )
   connect( m_writerJob, SIGNAL(writeSpeed(int, int)), this, SIGNAL(writeSpeed(int, int)) );
   connect( m_writerJob, SIGNAL(finished(bool)), this, SLOT(slotWriterJobFinished(bool)) );
   connect( m_writerJob, SIGNAL(newSubTask(const QString&)), this, SIGNAL(newSubTask(const QString&)) );
-  connect( m_writerJob, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+  connect( m_writerJob, SIGNAL(debuggingOutput(const QString&, const QString&)),
 	   this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
 }
 
 
 void K3bDataJob::setImager( K3bIsoImager* imager )
 {
-  m_isoImager = imager;
-  connect( m_isoImager, SIGNAL(sizeCalculated(int, int)), this, SLOT(slotSizeCalculationFinished(int, int)) );
+  if( m_isoImager != imager ) {
+    delete m_isoImager;
+
+    m_isoImager = imager;
+
+    connectImager();
+  }
+}
+
+
+void K3bDataJob::connectImager()
+{
+  m_isoImager->disconnect( this );
   connect( m_isoImager, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
   connect( m_isoImager, SIGNAL(percent(int)), this, SLOT(slotIsoImagerPercent(int)) );
   connect( m_isoImager, SIGNAL(finished(bool)), this, SLOT(slotIsoImagerFinished(bool)) );
-  connect( m_isoImager, SIGNAL(debuggingOutput(const QString&, const QString&)), 
+  connect( m_isoImager, SIGNAL(debuggingOutput(const QString&, const QString&)),
 	   this, SIGNAL(debuggingOutput(const QString&, const QString&)) );
 }
 
@@ -551,19 +628,15 @@ bool K3bDataJob::prepareWriterJob()
     if( d->usedDataMode == K3b::MODE1 )
       writer->addArgument( "-data" );
     else {
-      if( k3bcore->externalBinManager()->binObject("cdrecord") && 
+      if( k3bcore->externalBinManager()->binObject("cdrecord") &&
 	  k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "xamix" ) )
 	writer->addArgument( "-xa" );
       else
 	writer->addArgument( "-xa1" );
     }
 
-    if( d->doc->onTheFly() ) {
-      writer->addArgument( QString("-tsize=%1s").arg(m_isoImager->size()) )->addArgument("-");
-    }
-    else {
-      writer->addArgument( d->doc->tempDir() );
-    }
+    writer->addArgument( QString("-tsize=%1s").arg(m_isoImager->size()) )->addArgument("-");
+    writer->setForceNoEject( true );
 
     setWriterJob( writer );
   }
@@ -576,6 +649,7 @@ bool K3bDataJob::prepareWriterJob()
     // multisession
     writer->setMulti( d->usedMultiSessionMode == K3bDataDoc::START ||
 		      d->usedMultiSessionMode == K3bDataDoc::CONTINUE );
+    writer->setForceNoEject( true );
 
     // now write the tocfile
     if( d->tocFile ) delete d->tocFile;
@@ -593,10 +667,8 @@ bool K3bDataJob::prepareWriterJob()
 	*s << "\n";
 	*s << "TRACK MODE2_FORM1" << "\n";
       }
-      if( d->doc->onTheFly() )
-	*s << "DATAFILE \"-\" " << m_isoImager->size()*2048 << "\n";
-      else
-	*s << "DATAFILE \"" << d->doc->tempDir() << "\"\n";
+
+      *s << "DATAFILE \"-\" " << m_isoImager->size()*2048 << "\n";
 
       d->tocFile->close();
     }
@@ -616,7 +688,7 @@ bool K3bDataJob::prepareWriterJob()
 }
 
 
-void K3bDataJob::determineWritingMode()
+void K3bDataJob::prepareData()
 {
   // we don't need this when only creating image and it is possible
   // that the burn device is null
@@ -695,30 +767,36 @@ void K3bDataJob::determineWritingMode()
 
 void K3bDataJob::determineMultiSessionMode()
 {
+  //
+  // THIS IS ONLY CALLED IF d->doc->multiSessionMode() == K3bDataDoc::AUTO!
+  //
+
   if( d->doc->writingMode() == K3b::WRITING_MODE_AUTO ||
       d->doc->writingMode() == K3b::TAO ) {
     emit newSubTask( i18n("Searching for old session") );
-    
-    // 1. size of the project
-    // 2. size of the media
-    // 3. space left on appendable media >= project size (otherwise no ms)
-    // 4. size of the project >= size of the media*90% (fill up?)
-    
+
+    //
+    // Wait for the medium.
+    // In case an old session was imported we always want to continue or finish a multisession CD/DVD.
+    // Otherwise we wait for everything we could handle and decide what to do in
+    // determineMultiSessionMode( K3bDevice::DeviceHandler* ) below.
+    //
+
     int wantedMediaState = K3bDevice::STATE_INCOMPLETE|K3bDevice::STATE_EMPTY;
-    if( d->doc->size() != d->doc->burningSize() ) // this means that an old session has been imported.
+    if( d->doc->sessionImported() )
       wantedMediaState = K3bDevice::STATE_INCOMPLETE;
-    
-    int m = waitForMedia( d->doc->burner(), 
+
+    int m = waitForMedia( d->doc->burner(),
 			  wantedMediaState,
 			  K3bDevice::MEDIA_WRITABLE_CD );
-    
+
     if( m < 0 )
       cancel();
     else {
       // now we need to determine the media's size
-      connect( K3bDevice::sendCommand( K3bDevice::DeviceHandler::NG_DISKINFO, d->doc->burner() ), 
+      connect( K3bDevice::sendCommand( K3bDevice::DeviceHandler::NG_DISKINFO, d->doc->burner() ),
 	       SIGNAL(finished(K3bDevice::DeviceHandler*)),
-	       this, 
+	       this,
 	       SLOT(slotDetermineMultiSessionMode(K3bDevice::DeviceHandler*)) );
     }
   }
@@ -734,17 +812,37 @@ void K3bDataJob::determineMultiSessionMode()
 
 void K3bDataJob::slotDetermineMultiSessionMode( K3bDevice::DeviceHandler* dh )
 {
-  const K3bDevice::DiskInfo& info = dh->diskInfo();
+  //
+  // This is a little workaround for the bad cancellation handling in this job
+  // see cancel()
+  //
+  if( d->canceled ) {
+    if( active() ) {
+      cleanup();
+      jobFinished( false );
+    }
+  }
+  else {
+    d->usedMultiSessionMode = getMultiSessionMode( dh->diskInfo() );
 
+    // carry on with the writing
+    prepareWriting();
+  }
+}
+
+
+K3bDataDoc::MultiSessionMode K3bDataJob::getMultiSessionMode( const K3bDevice::DiskInfo& info )
+{
   if( info.appendable() ) {
     //
     // 3 cases:
     //  1. the project does not fit -> no multisession (resulting in asking for another media above)
-    //     Except if an old session has been imported.
     //  2. the project does fit and fills up the CD -> finish multisession
     //  3. the project does fit and does not fill up the CD -> continue multisession
     //
-    if( d->doc->size() > info.remainingSize().mode1Bytes() && d->doc->size() == d->doc->burningSize() )
+    // In case a session has been imported we do not consider NONE at all.
+    //
+    if( d->doc->size() > info.remainingSize().mode1Bytes() && !d->doc->sessionImported() )
       d->usedMultiSessionMode = K3bDataDoc::NONE;
     else if( d->doc->size() >= info.remainingSize().mode1Bytes()*9/10 )
       d->usedMultiSessionMode = K3bDataDoc::FINISH;
@@ -756,25 +854,26 @@ void K3bDataJob::slotDetermineMultiSessionMode( K3bDevice::DeviceHandler* dh )
     //
     // We only close the CD if the project fills up the CD almost completely (90%)
     //
-    if( d->doc->size() >= info.capacity().mode1Bytes()*9/10 )
+    if( d->doc->size() >= info.capacity().mode1Bytes()*9/10 ||
+	d->doc->writingMode() == K3b::DAO )
       d->usedMultiSessionMode = K3bDataDoc::NONE;
     else
       d->usedMultiSessionMode = K3bDataDoc::START;
   }
 
-  else { // complete (WE ACTUALLY NEVER GET HERE SINCE WE WAIT FOR AN EMPTY/APPENDABLE CD ABOVE!)
+  else { // complete (WE SHOULD ACTUALLY NEVER GET HERE SINCE WE WAIT FOR AN EMPTY/APPENDABLE CD ABOVE!)
     //
     // Now we decide only based on the project size.
     // let's just use a 680 MB CD as our reference
     //
-    if( d->doc->size()/1024/1024 >= 680*9/10 )
+    if( d->doc->size()/1024/1024 >= 680*9/10 ||
+	d->doc->writingMode() == K3b::DAO )
       d->usedMultiSessionMode = K3bDataDoc::NONE;
     else
       d->usedMultiSessionMode = K3bDataDoc::START;
   }
 
-  // carry on with the writing
-  prepareWriting();
+  return d->usedMultiSessionMode;
 }
 
 
@@ -789,36 +888,25 @@ void K3bDataJob::cancelAll()
   if( d->verificationJob )
     d->verificationJob->cancel();
 
-  // remove iso-image if it is unfinished or the user selected to remove image
-  if( QFile::exists( d->doc->tempDir() ) ) {
-    if( (!d->doc->onTheFly() || d->doc->onlyCreateImages() ) && (d->doc->removeImages() || !d->imageFinished) ) {
-      emit infoMessage( i18n("Removing ISO image %1").arg(d->doc->tempDir()), K3bJob::SUCCESS );
-      QFile::remove( d->doc->tempDir() );
-    }
-  }
+  d->pipe.close();
 
-  if( d->tocFile ) {
-    delete d->tocFile;
-    d->tocFile = 0;
-  }
-
-  // TODO: wait for the subjobs to be finished
-
-  emit finished(false);
+  cleanup();
 }
 
 
-void K3bDataJob::waitForDisk()
+bool K3bDataJob::waitForMedium()
 {
-  emit newSubTask( i18n("Waiting for media") );
-  if( waitForMedia( d->doc->burner(), 
+  emit newSubTask( i18n("Waiting for a medium") );
+  if( waitForMedia( d->doc->burner(),
 		    d->usedMultiSessionMode == K3bDataDoc::CONTINUE ||
 		    d->usedMultiSessionMode == K3bDataDoc::FINISH ?
 		    K3bDevice::STATE_INCOMPLETE :
 		    K3bDevice::STATE_EMPTY,
 		    K3bDevice::MEDIA_WRITABLE_CD ) < 0 ) {
-    cancel();
+    return false;
   }
+  else
+    return !d->canceled;
 }
 
 
@@ -845,11 +933,11 @@ QString K3bDataJob::jobDescription() const
 
 QString K3bDataJob::jobDetails() const
 {
-  if( d->doc->copies() > 1 && 
+  if( d->doc->copies() > 1 &&
       !d->doc->dummy() &&
       !(d->doc->multiSessionMode() == K3bDataDoc::CONTINUE ||
 	d->doc->multiSessionMode() == K3bDataDoc::FINISH) )
-    return i18n("ISO9660 Filesystem (Size: %1) - %n copy", 
+    return i18n("ISO9660 Filesystem (Size: %1) - %n copy",
 		"ISO9660 Filesystem (Size: %1) - %n copies",
 		d->doc->copies() )
       .arg(KIO::convertSize( d->doc->size() ));
@@ -858,4 +946,32 @@ QString K3bDataJob::jobDetails() const
       .arg(KIO::convertSize( d->doc->size() ));
 }
 
+
+K3bDataDoc::MultiSessionMode K3bDataJob::usedMultiSessionMode() const
+{
+  return d->usedMultiSessionMode;
+}
+
+
+void K3bDataJob::cleanup()
+{
+  if( !d->doc->onTheFly() && d->doc->removeImages() ) {
+    if( QFile::exists( d->doc->tempDir() ) ) {
+      d->imageFile.remove();
+      emit infoMessage( i18n("Removed image file %1").arg(d->doc->tempDir()), K3bJob::SUCCESS );
+    }
+  }
+
+  if( d->tocFile ) {
+    delete d->tocFile;
+    d->tocFile = 0;
+  }
+}
+
+
+bool K3bDataJob::hasBeenCanceled() const
+{
+  return d->canceled;
+}
+
 #include "k3bdatajob.moc"
