@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3biso9660.cpp 435652 2005-07-17 19:16:07Z trueg $
+ * $Id: k3biso9660.cpp 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,25 +13,16 @@
  * See the file "COPYING" for the exact licensing terms.
  */
 
+#include <config.h>
+#include <k3bglobals.h>
+
 #include "k3biso9660.h"
+#include "k3biso9660backend.h"
 
 #include <k3bdevice.h>
-#include <k3btoc.h>
-#include <k3btrack.h>
-#include <k3bmsf.h>
 
 #include "libisofs/isofs.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <errno.h>
-#include <fcntl.h>
-
 #include <qcstring.h>
 #include <qdir.h>
 #include <qfile.h>
@@ -39,13 +30,123 @@
 
 #include <kdebug.h>
 
-#ifndef O_LARGEFILE
-#define O_LARGEFILE 0
-#endif
+
+/* callback function for libisofs */
+int K3bIso9660::read_callback( char* buf, sector_t start, int len, void* udata ) 
+{
+  K3bIso9660* isoF = static_cast<K3bIso9660*>(udata);
+  
+  return isoF->read( start, buf, len );
+}
+
+/* callback function for libisofs */
+int K3bIso9660::isofs_callback( struct iso_directory_record *idr, void *udata ) 
+{
+  K3bIso9660 *iso = static_cast<K3bIso9660*> (udata);
+  QString path, isoPath,user,group,symlink;
+  int i;
+  int access;
+  int time,cdate,adate;
+  rr_entry rr;
+  bool special=false;
+  K3bIso9660Entry *entry=0;
+  //K3bIso9660Entry *oldentry=0;
+  char z_algo[2],z_params[2];
+  int z_size=0;
+
+   if (isonum_711(idr->name_len)==1) {
+     switch (idr->name[0]) {
+     case 0:
+      path+=(".");
+      special=true;
+      break;
+     case 1:
+      path+=("..");
+      special=true;
+      break;
+    }
+  }
+  //
+  // First extract the raw iso9660 name
+  //
+  if( !special ) {
+    for( i = 0; i < isonum_711( idr->name_len ); i++ ) {
+      if( idr->name[i] )
+	isoPath += idr->name[i];
+      }
+  }
+  else
+    isoPath = path;
+
+  //
+  // Now see if we have RockRidge
+  //
+  if( !iso->plainIso9660() && ParseRR(idr,&rr) > 0 ) {
+    iso->m_rr = true;
+    if (!special)
+      path = QString::fromLocal8Bit( rr.name );
+    symlink=rr.sl;
+    access=rr.mode;
+    time=0;//rr.st_mtime;
+    adate=0;//rr.st_atime;
+    cdate=0;//rr.st_ctime;
+    user.setNum(rr.uid);
+    group.setNum(rr.gid);
+    z_algo[0]=rr.z_algo[0];z_algo[1]=rr.z_algo[1];
+    z_params[0]=rr.z_params[0];z_params[1]=rr.z_params[1];
+    z_size=rr.z_size;
+  }
+  else {
+    access=iso->dirent->permissions() & ~S_IFMT;
+    adate=cdate=time=isodate_915(idr->date,0);
+    user=iso->dirent->user();
+    group=iso->dirent->group();
+    if (idr->flags[0] & 2) access |= S_IFDIR; else access |= S_IFREG;
+    if (!special) {
+      if( !iso->plainIso9660() && iso->jolietLevel() ) {
+	for (i=0;i<(isonum_711(idr->name_len)-1);i+=2) {
+	  QChar ch( be2me_16(*((ushort*)&(idr->name[i]))) );
+	  if (ch==';') break;
+	  path+=ch;
+	}
+      }
+      else {
+	// no RR, no Joliet, just plain iso9660
+	path = isoPath;
+
+	// remove the version field
+	int pos = path.find( ';' );
+	if( pos > 0 )
+	  path.truncate( pos );
+      }
+      if (path.endsWith(".")) path.setLength(path.length()-1);
+    }
+  }
+
+  if( !iso->plainIso9660() )
+    FreeRR(&rr);
+
+  if (idr->flags[0] & 2) {
+      entry = new K3bIso9660Directory( iso, isoPath, path, access | S_IFDIR, time, adate, cdate,
+				       user, group, symlink, 
+				       special ? 0 : isonum_733(idr->extent), 
+				       special ? 0 : isonum_733(idr->size) );
+  }
+  else {
+    entry = new K3bIso9660File( iso, isoPath, path, access, time, adate, cdate,
+			        user, group, symlink, isonum_733(idr->extent), isonum_733(idr->size) );
+    if (z_size)
+      (static_cast<K3bIso9660File*>(entry))->setZF( z_algo, z_params, z_size );
+  }
+  iso->dirent->addEntry(entry);
+  
+  return 0;
+}
 
 
 
 K3bIso9660Entry::K3bIso9660Entry( K3bIso9660* archive,
+				  const QString& isoName,
 				  const QString& name,
 				  int access,
 				  int date,
@@ -57,6 +158,7 @@ K3bIso9660Entry::K3bIso9660Entry( K3bIso9660* archive,
   : m_adate( adate ),
     m_cdate( cdate ),
     m_name( name ),
+    m_isoName( isoName ),
     m_date( date ),
     m_access( access ),
     m_user( user ),
@@ -77,6 +179,7 @@ K3bIso9660Entry::~K3bIso9660Entry()
 
 
 K3bIso9660File::K3bIso9660File( K3bIso9660* archive, 
+				const QString& isoName, 
 				const QString& name, 
 				int access,
 				int date, 
@@ -87,7 +190,7 @@ K3bIso9660File::K3bIso9660File( K3bIso9660* archive,
 				const QString& symlink, 
 				unsigned int pos, 
 				unsigned int size )
-  : K3bIso9660Entry( archive, name, access, date, adate, cdate, user, group, symlink ),
+  : K3bIso9660Entry( archive, isoName, name, access, date, adate, cdate, user, group, symlink ),
     m_startSector(pos),
     m_size(size)
 {
@@ -126,7 +229,8 @@ int K3bIso9660File::read( unsigned int pos, char* data, int maxlen ) const
     bufferLen = size() - pos + startSecOffset;
 
   // pad to 2048
-  bufferLen += (2048-(bufferLen%2048));
+  if( bufferLen%2048 )
+    bufferLen += (2048-(bufferLen%2048));
 
   // we need to buffer if we changed the startSec or need a bigger buffer
   if( startSecOffset || bufferLen > (unsigned int)maxlen ) {
@@ -161,7 +265,29 @@ int K3bIso9660File::read( unsigned int pos, char* data, int maxlen ) const
 }
 
 
+bool K3bIso9660File::copyTo( const QString& url ) const
+{
+  QFile of( url );
+  if( of.open( IO_WriteOnly ) ) {
+    char buffer[2048*10];
+    unsigned int pos = 0;
+    int r = 0;
+    while( ( r = read( pos, buffer, 2048*10 ) ) > 0 ) {
+      of.writeBlock( buffer, r );
+      pos += r;
+    }
+
+    return !r;
+  }
+  else {
+    kdDebug() << "(K3bIso9660File) could not open " << url << " for writing." << endl;
+    return false;
+  }
+}
+
+
 K3bIso9660Directory::K3bIso9660Directory( K3bIso9660* archive, 
+					  const QString& isoName, 
 					  const QString& name, 
 					  int access,
 					  int date, 
@@ -169,8 +295,13 @@ K3bIso9660Directory::K3bIso9660Directory( K3bIso9660* archive,
 					  int cdate, 
 					  const QString& user, 
 					  const QString& group,
-					  const QString& symlink )
-  : K3bIso9660Entry( archive, name, access, date, adate, cdate, user, group, symlink )
+					  const QString& symlink,
+					  unsigned int pos, 
+					  unsigned int size  )
+  : K3bIso9660Entry( archive, isoName, name, access, date, adate, cdate, user, group, symlink ),
+    m_bExpanded( size == 0 ), // we can only expand entries that represent an actual directory
+    m_startSector(pos),
+    m_size(size)
 {
   m_entries.setAutoDelete( true );
 }
@@ -180,8 +311,23 @@ K3bIso9660Directory::~K3bIso9660Directory()
 }
 
 
+void K3bIso9660Directory::expand()
+{
+  if( !m_bExpanded ) {
+    archive()->dirent = this;
+    if( ProcessDir( &K3bIso9660::read_callback, m_startSector, m_size, &K3bIso9660::isofs_callback, archive() ) )
+      kdDebug() << "(K3bIso9660) failed to expand dir: " << name() << " with size: " << m_size << endl;
+
+    m_bExpanded = true;
+  }
+}
+
+
 QStringList K3bIso9660Directory::entries() const
 {
+  // create a fake const method to fool the user ;)
+  const_cast<K3bIso9660Directory*>(this)->expand();
+
   QStringList l;
   
   QDictIterator<K3bIso9660Entry> it( m_entries );
@@ -192,11 +338,37 @@ QStringList K3bIso9660Directory::entries() const
 }
 
 
+QStringList K3bIso9660Directory::iso9660Entries() const
+{
+  // create a fake const method to fool the user ;)
+  const_cast<K3bIso9660Directory*>(this)->expand();
+
+  QStringList l;
+  
+  QDictIterator<K3bIso9660Entry> it( m_iso9660Entries );
+  for( ; it.current(); ++it )
+    l.append( it.currentKey() );
+  
+  return l;
+}
+
+
 K3bIso9660Entry* K3bIso9660Directory::entry( const QString& n )
 {
+  if( n.isEmpty() )
+    return 0;
+
+  expand();
+
   QString name(n);
+
+  // trailing slash ? -> remove
+  if( name.length() > 1 && name[name.length()-1] == '/' ) {
+    name.truncate( name.length()-1 );
+  }
+
   int pos = name.find( '/' );
-  if( pos == 0 ) {
+  while( pos == 0 ) {
     if( name.length() > 1 ) {
       name = name.mid( 1 ); // remove leading slash
       pos = name.find( '/' ); // look again
@@ -205,12 +377,6 @@ K3bIso9660Entry* K3bIso9660Directory::entry( const QString& n )
       return this;
   }
 
-  // trailing slash ? -> remove
-  if( pos != -1 && pos == (int)name.length()-1 ) {
-    name = name.left( pos );
-    pos = name.find( '/' ); // look again
-  }
-
   if ( pos != -1 ) {
     QString left = name.left( pos );
     QString right = name.mid( pos + 1 );
@@ -225,15 +391,60 @@ K3bIso9660Entry* K3bIso9660Directory::entry( const QString& n )
 }
 
 
+K3bIso9660Entry* K3bIso9660Directory::iso9660Entry( const QString& n )
+{
+  if( n.isEmpty() )
+    return 0;
+
+  expand();
+
+  QString name(n);
+
+  // trailing slash ? -> remove
+  if( name.length() > 1 && name[name.length()-1] == '/' ) {
+    name.truncate( name.length()-1 );
+  }
+
+  int pos = name.find( '/' );
+  while( pos == 0 ) {
+    if( name.length() > 1 ) {
+      name = name.mid( 1 ); // remove leading slash
+      pos = name.find( '/' ); // look again
+    }
+    else // "/"
+      return this;
+  }
+
+  if ( pos != -1 ) {
+    QString left = name.left( pos );
+    QString right = name.mid( pos + 1 );
+
+    K3bIso9660Entry* e = m_iso9660Entries[ left ];
+    if ( !e || !e->isDirectory() )
+      return 0;
+    return static_cast<K3bIso9660Directory*>(e)->iso9660Entry( right );
+  }
+
+  return m_iso9660Entries[ name ];
+}
+
+
 const K3bIso9660Entry* K3bIso9660Directory::entry( const QString& name ) const
 {
   return const_cast<K3bIso9660Directory*>(this)->entry( name );
 }
 
 
+const K3bIso9660Entry* K3bIso9660Directory::iso9660Entry( const QString& name ) const
+{
+  return const_cast<K3bIso9660Directory*>(this)->iso9660Entry( name );
+}
+
+
 void K3bIso9660Directory::addEntry( K3bIso9660Entry* entry )
 {
   m_entries.insert( entry->name(), entry );
+  m_iso9660Entries.insert( entry->isoName(), entry );
 }
 
 
@@ -246,12 +457,12 @@ public:
   Private() 
     : cdDevice(0),
       fd(-1),
-      closeFd(false),
-      startSector(0) {
+      isOpen(false),
+      startSector(0),
+      plainIso9660(false),
+      backend(0) {
   }
 
-  bool deleteDev;
-
   QPtrList<K3bIso9660Directory> elToritoDirs;
   QPtrList<K3bIso9660Directory> jolietDirs;
   QPtrList<K3bIso9660Directory> isoDirs;
@@ -261,10 +472,15 @@ public:
 
   K3bDevice::Device* cdDevice;
   int fd;
-  bool closeFd;
+
+  bool isOpen;
 
   // only used for direkt K3bDevice::Device access
   unsigned int startSector;
+
+  bool plainIso9660;
+
+  K3bIso9660Backend* backend;
 };
 
 
@@ -282,6 +498,12 @@ K3bIso9660::K3bIso9660( int fd )
 }
 
 
+K3bIso9660::K3bIso9660( K3bIso9660Backend* backend )
+{
+  d = new Private();
+  d->backend = backend;
+}
+
 
 K3bIso9660::K3bIso9660( K3bDevice::Device* dev, unsigned int startSector )
 {
@@ -294,156 +516,35 @@ K3bIso9660::K3bIso9660( K3bDevice::Device* dev, unsigned int startSector )
 K3bIso9660::~K3bIso9660()
 {
   close();
+  delete d->backend;
   delete d;
 }
 
 
+void K3bIso9660::setStartSector( unsigned int startSector )
+{
+  d->startSector = startSector;
+}
+
+
+void K3bIso9660::setPlainIso9660( bool b )
+{
+  d->plainIso9660 = b;
+}
+
+
+bool K3bIso9660::plainIso9660() const
+{
+  return d->plainIso9660;
+}
+
+
 int K3bIso9660::read( unsigned int sector, char* data, int count )
 {
   if( count == 0 )
     return 0;
-
-  int read = -1;
-
-  if( d->cdDevice != 0 ) {
-
-    int retries = 10;  // TODO: no fixed value
-    while( retries && !d->cdDevice->read10( (unsigned char*)data,
-					    count*2048,
-					    sector,
-					    count ) )
-      retries--;
-
-    if( retries > 0 )
-      read = count;
-
-  
-#ifdef Q_OS_LINUX
-    // fallback
-    if( read < 0 ) {
-      kdDebug() << "(K3bIso9660) falling back to stdlib read" << endl;
-      if( ::lseek( d->cdDevice->handle(), static_cast<unsigned long long>(sector)*2048, SEEK_SET ) == -1 )
-	kdDebug() << "(K3bIso9660) seek failed." << endl;
-      else {
-	read = ::read( d->cdDevice->handle(), data, count*2048 );
-	if( read < 0 )
-	  kdDebug() << "(K3bIso9660) stdlib read failed." << endl;
-	else
-	  read /= 2048;
-      }
-    }
-#endif
-
-    return read;
-  }
-  else {
-    if( ::lseek( d->fd, static_cast<unsigned long long>(sector)*2048, SEEK_SET ) != -1 )
-      if( (read = ::read( d->fd, data, count*2048 )) != -1 )
-	return read / 2048;
-
-    return -1;
-  }
-}
-
-
-/* callback function for libisofs */
-static int readf( char* buf, sector_t start, int len, void* udata ) 
-{
-  K3bIso9660* isoF = static_cast<K3bIso9660*>(udata);
-  
-  return isoF->read( start, buf, len );
-}
-
-/* callback function for libisofs */
-static int mycallb(struct iso_directory_record *idr,void *udata) 
-{
-  K3bIso9660 *iso = static_cast<K3bIso9660*> (udata);
-  QString path,user,group,symlink;
-  int i;
-  int access;
-  int time,cdate,adate;
-  rr_entry rr;
-  bool special=false;
-  K3bIso9660Entry *entry=0,*oldentry=0;
-  char z_algo[2],z_params[2];
-  int z_size=0;
-
-  if (iso->level) {
-    if (isonum_711(idr->name_len)==1) {
-      switch (idr->name[0]) {
-      case 0:
-	path+=(".");
-	special=true;
-	break;
-      case 1:
-	path+=("..");
-	special=true;
-	break;
-      }
-    }
-    if(ParseRR(idr,&rr)>0) {
-      iso->m_rr = true;
-      if (!special)
-	path=rr.name;
-      symlink=rr.sl;
-      access=rr.mode;
-      time=0;//rr.st_mtime;
-      adate=0;//rr.st_atime;
-      cdate=0;//rr.st_ctime;
-      user.setNum(rr.uid);
-      group.setNum(rr.gid);
-      z_algo[0]=rr.z_algo[0];z_algo[1]=rr.z_algo[1];
-      z_params[0]=rr.z_params[0];z_params[1]=rr.z_params[1];
-      z_size=rr.z_size;
-    } else {
-      access=iso->dirent->permissions() & ~S_IFMT;
-      adate=cdate=time=isodate_915(idr->date,0);
-      user=iso->dirent->user();
-      group=iso->dirent->group();
-      if (idr->flags[0] & 2) access |= S_IFDIR; else access |= S_IFREG;
-      if (!special) {
-	if (iso->jolietLevel()) {
-	  for (i=0;i<(isonum_711(idr->name_len)-1);i+=2) {
-	    QChar ch( be2me_16(*((ushort*)&(idr->name[i]))) );
-	    if (ch==';') break;
-	    path+=ch;
-	  }
-	} else {
-	  for (i=0;i<isonum_711(idr->name_len);i++) {
-	    if (idr->name[i]==';') break;
-	    if (idr->name[i]) path+=(idr->name[i]);
-	  }
-	}
-	if (path.endsWith(".")) path.setLength(path.length()-1);
-      }
-    }
-    FreeRR(&rr);
-    if (idr->flags[0] & 2) {
-      entry = new K3bIso9660Directory( iso, path, access | S_IFDIR, time, adate, cdate,
-				       user, group, symlink );
-    } else {
-//       kdDebug() << "(K3bIso9660) creating file: " << path << " start: " << isonum_733(idr->extent)
-// 		<< " size: " << isonum_733(idr->size) << endl;
-      entry = new K3bIso9660File( iso, path, access, time, adate, cdate,
-				  user, group, symlink, isonum_733(idr->extent), isonum_733(idr->size) );
-      if (z_size)
-	(static_cast<K3bIso9660File*>(entry))->setZF( z_algo, z_params, z_size );
-    }
-    iso->dirent->addEntry(entry);
-  }
-
-  if ( (idr->flags[0] & 2) && (iso->level==0 || !special) ) {
-    if (iso->level) {
-      oldentry=iso->dirent;
-      iso->dirent=static_cast<K3bIso9660Directory*> (entry);
-    }
-    iso->level++;
-    ProcessDir( &readf, isonum_733(idr->extent),isonum_733(idr->size),&mycallb,udata);
-    iso->level--;
-    if(iso->level)
-      iso->dirent = static_cast<K3bIso9660Directory*>(oldentry);
-  }
-  return 0;
+  else
+    return d->backend->read( sector, data, count );
 }
 
 
@@ -455,23 +556,23 @@ void K3bIso9660::addBoot(struct el_torito_boot_descriptor* bootdesc)
   QString path;
   K3bIso9660File *entry;
     
-  entry=new K3bIso9660File( this, "Catalog", dirent->permissions() & ~S_IFDIR,
+  entry=new K3bIso9660File( this, "Catalog", "Catalog", dirent->permissions() & ~S_IFDIR,
 			    dirent->date(), dirent->adate(), dirent->cdate(),
 			    dirent->user(), dirent->group(), QString::null,
 			    isonum_731(bootdesc->boot_catalog), 2048 );
   dirent->addEntry(entry);
-  if (!ReadBootTable(&readf,isonum_731(bootdesc->boot_catalog),&boot,this)) {
+  if (!ReadBootTable(&K3bIso9660::read_callback,isonum_731(bootdesc->boot_catalog),&boot,this)) {
     i=1;
     be=boot.defentry;
     while (be) {
-      size=BootImageSize(&readf,
+      size=BootImageSize(&K3bIso9660::read_callback,
 			 isonum_711(((struct default_entry*) be->data)->media),
 			 isonum_731(((struct default_entry*) be->data)->start),
 			 isonum_721(((struct default_entry*) be->data)->seccount),
 			 this);
       path="Default Image";
       if (i>1) path += " (" + QString::number(i) + ")";
-      entry=new K3bIso9660File( this, path, dirent->permissions() & ~S_IFDIR,
+      entry=new K3bIso9660File( this, path, path, dirent->permissions() & ~S_IFDIR,
 				dirent->date(), dirent->adate(), dirent->cdate(),
 				dirent->user(), dirent->group(), QString::null,
 				isonum_731(((struct default_entry*) be->data)->start), size<<9 );
@@ -487,26 +588,44 @@ void K3bIso9660::addBoot(struct el_torito_boot_descriptor* bootdesc)
 
 bool K3bIso9660::open()
 {
-  // open the file
-  if( !m_filename.isEmpty() ) {
-    d->fd = ::open( QFile::encodeName( m_filename ), O_RDONLY|O_LARGEFILE );
-    if( d->fd < 0 )
-      return false;
-    d->closeFd = true;
-  }
-  else if( d->cdDevice ) {
-    if( !d->cdDevice->open() )
-      return false;
+  if( d->isOpen )
+    return true;
 
-    // set optimal reading speed
-    d->cdDevice->setSpeed( 0xffff, 0xffff );
+  if( !d->backend ) {
+    // create a backend
+
+    if( !m_filename.isEmpty() )
+      d->backend = new K3bIso9660FileBackend( m_filename );
+
+    else if( d->fd > 0 )
+      d->backend = new K3bIso9660FileBackend( d->fd );
+
+    else if( d->cdDevice ) {
+      // now check if we have a scrambled video dvd
+      if( d->cdDevice->copyrightProtectionSystemType() > 0 ) {
+      	
+	kdDebug() << "(K3bIso9660) found encrypted dvd. using libdvdcss." << endl;
+	
+	// open the libdvdcss stuff
+	d->backend = new K3bIso9660LibDvdCssBackend( d->cdDevice );
+	if( !d->backend->open() ) {
+	  // fallback to devicebackend
+	  delete d->backend;
+	  d->backend = new K3bIso9660DeviceBackend( d->cdDevice );
+	}
+      }
+      else
+	d->backend = new K3bIso9660DeviceBackend( d->cdDevice );
+    }
   }
-  else if( d->fd < 0 )
+
+  d->isOpen = d->backend->open();
+  if( !d->isOpen )
     return false;
 
   iso_vol_desc *desc;
   QString path,tmp,uid,gid;
-  struct stat buf;
+  k3b_struct_stat buf;
   int access,c_i,c_j;
   struct el_torito_boot_descriptor* bootdesc;
 
@@ -514,9 +633,9 @@ bool K3bIso9660::open()
   /* We'll use the permission and user/group of the 'host' file except
    * in Rock Ridge, where the permissions are stored on the file system
    */
-  if (::stat( m_filename.local8Bit(), &buf )<0) {
+  if ( k3b_stat( QFile::encodeName(m_filename), &buf ) < 0 ) {
     /* defaults, if stat fails */
-    memset(&buf,0,sizeof(struct stat));
+    memset(&buf,0,sizeof(k3b_struct_stat));
     buf.st_mode=0777;
   }
   uid.setNum(buf.st_uid);
@@ -527,10 +646,11 @@ bool K3bIso9660::open()
   int c_b=1;
   c_i=1;c_j=1;
 
-  desc = ReadISO9660( &readf, d->startSector, this );
+  desc = ReadISO9660( &K3bIso9660::read_callback, d->startSector, this );
 
   if (!desc) {
     kdDebug() << "K3bIso9660::openArchive no volume descriptors" << endl;
+    close();
     return false;
   }
 
@@ -547,7 +667,7 @@ bool K3bIso9660::open()
 	if( c_b > 1 )
 	  path += " (" + QString::number(c_b) + ")";
                         
-	dirent = new K3bIso9660Directory( this, path, access | S_IFDIR,
+	dirent = new K3bIso9660Directory( this, path, path, access | S_IFDIR,
 					  buf.st_mtime, buf.st_atime, buf.st_ctime, uid, gid, QString::null );
 	d->elToritoDirs.append( dirent );
                         
@@ -565,6 +685,11 @@ bool K3bIso9660::open()
 	struct iso_directory_record* idr = (struct iso_directory_record*)&primaryDesc->root_directory_record;
 
 	m_joliet = JolietLevel(&desc->data);
+
+	// skip joliet in plain iso mode
+	if( m_joliet && plainIso9660() )
+	  break;
+
 	if (m_joliet) {
 	  path = "Joliet level " + QString::number(m_joliet);
 	  if( c_j > 1 )
@@ -576,11 +701,12 @@ bool K3bIso9660::open()
 	    path += " (" + QString::number(c_i) + ")";
 	}
 	
-	dirent = new K3bIso9660Directory( this, path, access | S_IFDIR,
+	dirent = new K3bIso9660Directory( this, path, path, access | S_IFDIR,
 					  buf.st_mtime, buf.st_atime, buf.st_ctime, uid, gid, QString::null );
 
-	level=0;
-	mycallb( idr, this );
+	// expand the root entry
+	ProcessDir( &K3bIso9660::read_callback, isonum_733(idr->extent),isonum_733(idr->size),&K3bIso9660::isofs_callback,this);
+	
 	if (m_joliet)
 	  c_j++;
 	else
@@ -606,6 +732,12 @@ bool K3bIso9660::open()
 }
 
 
+bool K3bIso9660::isOpen() const
+{
+  return d->isOpen;
+}
+
+
 void K3bIso9660::createSimplePrimaryDesc( struct iso_primary_descriptor* desc )
 {
   d->primaryDesc.volumeId = QString::fromLocal8Bit( desc->volume_id, 32 ).stripWhiteSpace();
@@ -623,24 +755,22 @@ void K3bIso9660::createSimplePrimaryDesc( struct iso_primary_descriptor* desc )
 
 void K3bIso9660::close()
 {
-  if( d->closeFd ) {
-    ::close( d->fd );
-    d->closeFd = false;
-    d->fd = -1;
+  if( d->isOpen ) {
+    d->backend->close();
+
+    // Since the first isoDir is the KArchive
+    // root we must not delete it but all the
+    // others.
+    
+    d->elToritoDirs.setAutoDelete(true);
+    d->jolietDirs.setAutoDelete(true);
+    d->isoDirs.setAutoDelete(true);
+    d->elToritoDirs.clear();
+    d->jolietDirs.clear();
+    d->isoDirs.clear();
+
+    d->isOpen = false;
   }
-  else if( d->cdDevice )
-    d->cdDevice->close();
-
-  // Since the first isoDir is the KArchive
-  // root we must not delete it but all the
-  // others.
-
-  d->elToritoDirs.setAutoDelete(true);
-  d->jolietDirs.setAutoDelete(true);
-  d->isoDirs.setAutoDelete(true);
-  d->elToritoDirs.clear();
-  d->jolietDirs.clear();
-  d->isoDirs.clear();
 }
 
 
@@ -676,29 +806,94 @@ const K3bIso9660SimplePrimaryDescriptor& K3bIso9660::primaryDescriptor() const
 
 void K3bIso9660::debug() const
 {
-  kdDebug() << "System Id:         " << primaryDescriptor().systemId << endl;
-  kdDebug() << "Volume Id:         " << primaryDescriptor().volumeId << endl;
-  kdDebug() << "Volume Set Id:     " << primaryDescriptor().volumeSetId << endl;
-  kdDebug() << "Preparer Id:       " << primaryDescriptor().preparerId << endl;
-  kdDebug() << "Publisher Id:      " << primaryDescriptor().publisherId << endl;
-  kdDebug() << "Application Id:    " << primaryDescriptor().applicationId << endl;
-  kdDebug() << "Volume Set Size:   " << primaryDescriptor().volumeSetSize << endl;
-  kdDebug() << "Volume Set Number: " << primaryDescriptor().volumeSetNumber << endl;
+  if( isOpen() ) {
+    kdDebug() << "System Id:         " << primaryDescriptor().systemId << endl;
+    kdDebug() << "Volume Id:         " << primaryDescriptor().volumeId << endl;
+    kdDebug() << "Volume Set Id:     " << primaryDescriptor().volumeSetId << endl;
+    kdDebug() << "Preparer Id:       " << primaryDescriptor().preparerId << endl;
+    kdDebug() << "Publisher Id:      " << primaryDescriptor().publisherId << endl;
+    kdDebug() << "Application Id:    " << primaryDescriptor().applicationId << endl;
+    kdDebug() << "Volume Set Size:   " << primaryDescriptor().volumeSetSize << endl;
+    kdDebug() << "Volume Set Number: " << primaryDescriptor().volumeSetNumber << endl;
 
-  debugEntry( firstIsoDirEntry(), 0 );
+    if( firstIsoDirEntry() ) {
+      kdDebug() << "First ISO Dir entry:" << endl;
+      kdDebug() << "----------------------------------------------" << endl;
+      debugEntry( firstIsoDirEntry(), 0 );
+      kdDebug() << "----------------------------------------------" << endl << endl;
+    }
+    if( firstRRDirEntry() ) {
+      kdDebug() << "First RR Dir entry:" << endl;
+      kdDebug() << "----------------------------------------------" << endl;
+      debugEntry( firstRRDirEntry(), 0 );
+      kdDebug() << "----------------------------------------------" << endl << endl;
+    }
+    if( firstJolietDirEntry() ) {
+      kdDebug() << "First Joliet Dir entry:" << endl;
+      kdDebug() << "----------------------------------------------" << endl;
+      debugEntry( firstJolietDirEntry(), 0 );
+      kdDebug() << "----------------------------------------------" << endl << endl;
+    }
+  }
 }
 
 
 void K3bIso9660::debugEntry( const K3bIso9660Entry* entry, int depth ) const
 {
+  if( !entry ) {
+    kdDebug() << "(K3bIso9660::debugEntry) null entry." << endl;
+    return;
+  }
+
   QString spacer;
   spacer.fill( ' ', depth*3 );
-
-  kdDebug() << spacer << "- " << entry->name() << endl;
+  kdDebug() << spacer << "- " << entry->name() << " (" << entry->isoName() << ")" << endl;
   if( entry->isDirectory() ) {
     const K3bIso9660Directory* dir = dynamic_cast<const K3bIso9660Directory*>(entry);
     QStringList entries = dir->entries();
-    for( QStringList::const_iterator it = entries.begin(); it != entries.end(); ++it )
+    for( QStringList::const_iterator it = entries.begin(); it != entries.end(); ++it ) {
       debugEntry( dir->entry( *it ), depth+1 );
+    }
   }
 }
+
+
+K3bIso9660SimplePrimaryDescriptor::K3bIso9660SimplePrimaryDescriptor()
+  : volumeSetSize(0),
+    volumeSetNumber(0),
+    logicalBlockSize(0),
+    volumeSpaceSize(0)
+{
+}
+
+
+bool operator==( const K3bIso9660SimplePrimaryDescriptor& d1,
+		 const K3bIso9660SimplePrimaryDescriptor& d2 )
+{
+  return( d1.volumeId == d2.volumeId &&
+	  d1.systemId == d2.systemId &&
+	  d1.volumeSetId == d2.volumeSetId &&
+	  d1.publisherId == d2.publisherId &&
+	  d1.preparerId == d2.preparerId &&
+	  d1.applicationId == d2.applicationId &&
+	  d1.volumeSetSize == d2.volumeSetSize &&
+	  d1.volumeSetNumber == d2.volumeSetNumber &&
+	  d1.logicalBlockSize == d2.logicalBlockSize &&
+	  d1.volumeSpaceSize == d2.volumeSpaceSize );
+}
+
+
+bool operator!=( const K3bIso9660SimplePrimaryDescriptor& d1,
+		 const K3bIso9660SimplePrimaryDescriptor& d2 )
+{
+  return( d1.volumeId != d2.volumeId ||
+	  d1.systemId != d2.systemId ||
+	  d1.volumeSetId != d2.volumeSetId ||
+	  d1.publisherId != d2.publisherId ||
+	  d1.preparerId != d2.preparerId ||
+	  d1.applicationId != d2.applicationId ||
+	  d1.volumeSetSize != d2.volumeSetSize ||
+	  d1.volumeSetNumber != d2.volumeSetNumber ||
+	  d1.logicalBlockSize != d2.logicalBlockSize ||
+	  d1.volumeSpaceSize != d2.volumeSpaceSize );
+}
