@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3btempdirselectionwidget.cpp 507450 2006-02-09 09:01:24Z trueg $
+ * $Id: k3btempdirselectionwidget.cpp 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include <kiconloader.h>
 #include <kurlrequester.h>
 #include <kio/global.h>
+#include <kconfig.h>
 
 
 K3bTempDirSelectionWidget::K3bTempDirSelectionWidget( QWidget *parent, const char *name )
@@ -83,14 +84,26 @@ K3bTempDirSelectionWidget::~K3bTempDirSelectionWidget()
 }
 
 
-void K3bTempDirSelectionWidget::slotFreeTempSpace(const QString&,
-						  unsigned long,
-						  unsigned long,
-						  unsigned long kbAvail)
+unsigned long K3bTempDirSelectionWidget::freeTempSpace() const
 {
-  m_labelFreeSpace->setText( KIO::convertSizeFromKB(kbAvail) );
+  QString path = m_editDirectory->url();
 
-  m_freeTempSpace = kbAvail;
+  if( !QFile::exists( path ) )
+    path.truncate( path.findRev('/') );
+
+  unsigned long size;
+  K3b::kbFreeOnFs( path, size, m_freeTempSpace );
+  
+  return m_freeTempSpace;
+}
+
+
+void K3bTempDirSelectionWidget::slotUpdateFreeTempSpace()
+{
+  // update the temp space
+  freeTempSpace();
+
+  m_labelFreeSpace->setText( KIO::convertSizeFromKB(m_freeTempSpace) );
 
   if( m_labelCdSize ) {
     if( m_freeTempSpace < m_requestedSize/1024 )
@@ -102,21 +115,6 @@ void K3bTempDirSelectionWidget::slotFreeTempSpace(const QString&,
 }
 
 
-void K3bTempDirSelectionWidget::slotUpdateFreeTempSpace()
-{
-  QString path = m_editDirectory->url();
-
-  if( !QFile::exists( path ) )
-    path.truncate( path.findRev('/') );
-
-  unsigned long size, avail;
-  if( K3b::kbFreeOnFs( path, size, avail ) )
-    slotFreeTempSpace( path, size, 0, avail );
-  else
-    m_labelFreeSpace->setText("-");
-}
-
-
 void K3bTempDirSelectionWidget::slotTempDirButtonPressed( KURLRequester* r )
 {
   // set the correct mode for the filedialog
@@ -218,10 +216,20 @@ void K3bTempDirSelectionWidget::setNeededSize( KIO::filesize_t bytes )
 
 void K3bTempDirSelectionWidget::saveConfig()
 {
-  QString oldGroup = k3bcore->config()->group();
-  k3bcore->config()->setGroup( "General Options" );
-  k3bcore->config()->writePathEntry( "Temp Dir", tempDirectory() );
-  k3bcore->config()->setGroup( oldGroup );
+  KConfigGroup grp( k3bcore->config(), "General Options" );
+  grp.writePathEntry( "Temp Dir", tempDirectory() );
+}
+
+
+void K3bTempDirSelectionWidget::readConfig( KConfigBase* c )
+{
+  setTempPath( c->readPathEntry( "image path", K3b::defaultTempPath() ) );
+}
+
+
+void K3bTempDirSelectionWidget::saveConfig( KConfigBase* c )
+{
+  c->writePathEntry( "image path", tempPath() );
 }
 
 #include "k3btempdirselectionwidget.moc"
