@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bdvdcopydialog.cpp 471081 2005-10-16 10:55:47Z trueg $
+ * $Id: k3bdvdcopydialog.cpp 660354 2007-05-02 12:46:21Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,14 +20,16 @@
 #include <k3bwriterselectionwidget.h>
 #include <k3bglobals.h>
 #include <k3bstdguiitems.h>
-#include <k3bdevicecombobox.h>
+#include <k3bmediaselectioncombobox.h>
 #include <k3bdevice.h>
 #include <k3bdevicemanager.h>
-#include <k3bcore.h>
 #include <k3bexternalbinmanager.h>
 #include <k3bburnprogressdialog.h>
 #include <k3bwritingmodewidget.h>
 #include <k3bthememanager.h>
+#include <k3bapplication.h>
+#include <k3bmediacache.h>
+#include <k3biso9660.h>
 
 #include <qlayout.h>
 #include <qgroupbox.h>
@@ -67,18 +69,21 @@ K3bDvdCopyDialog::K3bDvdCopyDialog( QWidget* parent, const char* name, bool moda
   //
   // Source group
   // //////////////////////////////////////////////////////////////////////////
-  QGroupBox* groupSource = new QGroupBox( 1, Qt::Vertical, i18n("DVD Reader Device"), w );
+  QGroupBox* groupSource = new QGroupBox( 1, Qt::Vertical, i18n("Source Medium"), w );
   groupSource->setInsideSpacing( spacingHint() );
   groupSource->setInsideMargin( marginHint() );
-  m_comboSourceDevice = new K3bDeviceComboBox( groupSource );
-  m_comboSourceDevice->addDevices( k3bcore->deviceManager()->dvdReader() );
+  m_comboSourceDevice = new K3bMediaSelectionComboBox( groupSource );
+  m_comboSourceDevice->setWantedMediumType( K3bDevice::MEDIA_WRITABLE_DVD|K3bDevice::MEDIA_DVD_ROM );
+  m_comboSourceDevice->setWantedMediumState( K3bDevice::STATE_COMPLETE|K3bDevice::STATE_INCOMPLETE );
   // //////////////////////////////////////////////////////////////////////////
 
   //
   // Writer group
   // //////////////////////////////////////////////////////////////////////////
-  m_writerSelectionWidget = new K3bWriterSelectionWidget( true, w );
+  m_writerSelectionWidget = new K3bWriterSelectionWidget( w );
   m_writerSelectionWidget->setSupportedWritingApps( K3b::GROWISOFS );
+  m_writerSelectionWidget->setWantedMediumType( K3bDevice::MEDIA_WRITABLE_DVD );
+  m_writerSelectionWidget->setWantedMediumState( K3bDevice::STATE_EMPTY );
   // //////////////////////////////////////////////////////////////////////////
 
   //
@@ -96,13 +101,14 @@ K3bDvdCopyDialog::K3bDvdCopyDialog( QWidget* parent, const char* name, bool moda
   groupWritingMode->setInsideMargin( marginHint() );
   m_writingModeWidget = new K3bWritingModeWidget( groupWritingMode );
 
-  QGroupBox* groupOptions = new QGroupBox( 4, Qt::Vertical, i18n("Options"), optionTab );
+  QGroupBox* groupOptions = new QGroupBox( 5, Qt::Vertical, i18n("Settings"), optionTab );
   groupOptions->setInsideSpacing( spacingHint() );
   groupOptions->setInsideMargin( marginHint() );
   m_checkSimulate = K3bStdGuiItems::simulateCheckbox( groupOptions );
-  m_checkOnTheFly = K3bStdGuiItems::onTheFlyCheckbox( groupOptions );
+  m_checkCacheImage = K3bStdGuiItems::createCacheImageCheckbox( groupOptions );
   m_checkOnlyCreateImage = K3bStdGuiItems::onlyCreateImagesCheckbox( groupOptions );
   m_checkDeleteImages = K3bStdGuiItems::removeImagesCheckbox( groupOptions );
+  m_checkVerifyData = K3bStdGuiItems::verifyCheckBox( groupOptions );
 
   QGroupBox* groupCopies = new QGroupBox( 2, Qt::Horizontal, i18n("Copies"), optionTab );
   groupCopies->setInsideSpacing( spacingHint() );
@@ -186,14 +192,13 @@ K3bDvdCopyDialog::K3bDvdCopyDialog( QWidget* parent, const char* name, bool moda
   // //////////////////////////////////////////////////////////////////////////
   connect( m_writerSelectionWidget, SIGNAL(writerChanged()), this, SLOT(slotToggleAll()) );
   connect( m_comboSourceDevice, SIGNAL(selectionChanged(K3bDevice::Device*)), this, SLOT(slotToggleAll()) );
+  connect( m_comboSourceDevice, SIGNAL(selectionChanged(K3bDevice::Device*)), 
+	   this, SLOT(slotSourceMediumChanged(K3bDevice::Device*)) );
   connect( m_checkSimulate, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
-  connect( m_checkOnTheFly, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
+  connect( m_checkCacheImage, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
   connect( m_checkOnlyCreateImage, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
   connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(slotToggleAll()) );
 
-  connect( k3bcore->deviceManager(), SIGNAL(changed(K3bDevice::DeviceManager*)),
-	   this, SLOT(slotDeviceManagerChanged(K3bDevice::DeviceManager*)) );
-
   QToolTip::add( m_checkIgnoreReadErrors, i18n("Skip unreadable sectors") );
   QWhatsThis::add( m_checkIgnoreReadErrors, i18n("<p>If this option is checked and K3b is not able to read a sector from the "
 						 "source CD/DVD it will be replaced with zeros on the resulting copy.") );
@@ -207,24 +212,38 @@ K3bDvdCopyDialog::~K3bDvdCopyDialog()
 
 void K3bDvdCopyDialog::init()
 {
-  if( !m_writerSelectionWidget->writerDevice() )
-    m_checkOnlyCreateImage->setChecked( true );
+  slotSourceMediumChanged( m_comboSourceDevice->selectedDevice() );
+}
+
+
+void K3bDvdCopyDialog::setReadingDevice( K3bDevice::Device* dev )
+{
+  m_comboSourceDevice->setSelectedDevice( dev );
+}
+
+
+K3bDevice::Device* K3bDvdCopyDialog::readingDevice() const
+{
+  return m_comboSourceDevice->selectedDevice();
 }
 
 
 void K3bDvdCopyDialog::slotStartClicked()
 {
   //
-  // check for m_tempDirSelectionWidget->tempPath()
+  // Let's check the available size
   //
-  if( !m_checkOnlyCreateImage->isChecked() && !m_checkOnTheFly->isChecked() ) 
-    if( QFile::exists( m_tempDirSelectionWidget->tempPath() ) ) {
-      if( KMessageBox::warningYesNo( this,
-				     i18n("Do you want to overwrite %1?").arg(m_tempDirSelectionWidget->tempPath()),
-				     i18n("File Exists") )
-	  != KMessageBox::Yes )
+  if( m_checkCacheImage->isChecked() || m_checkOnlyCreateImage->isChecked() ) {
+    if( neededSize()/1024 > m_tempDirSelectionWidget->freeTempSpace() ) {
+      if( KMessageBox::warningContinueCancel( this, i18n("There seems to be not enough free space in temporary directory. "
+							 "Write anyway?") ) == KMessageBox::Cancel )
 	return;
     }
+  }
+
+  //
+  // The job asks if we want to overwrite.
+  //
 
   K3bJobProgressDialog* dlg = 0;
   if( m_checkOnlyCreateImage->isChecked() ) {
@@ -239,20 +258,30 @@ void K3bDvdCopyDialog::slotStartClicked()
   job->setWriterDevice( m_writerSelectionWidget->writerDevice() );
   job->setReaderDevice( m_comboSourceDevice->selectedDevice() );
   job->setImagePath( m_tempDirSelectionWidget->tempPath() );
-  job->setRemoveImageFiles( m_checkDeleteImages->isChecked() );
+  job->setRemoveImageFiles( m_checkDeleteImages->isChecked() && !m_checkOnlyCreateImage->isChecked() );
   job->setOnlyCreateImage( m_checkOnlyCreateImage->isChecked() );
   job->setSimulate( m_checkSimulate->isChecked() );
-  job->setOnTheFly( m_checkOnTheFly->isChecked() );
+  job->setOnTheFly( !m_checkCacheImage->isChecked() );
   job->setWriteSpeed( m_writerSelectionWidget->writerSpeed() );
   job->setCopies( m_checkSimulate->isChecked() ? 1 : m_spinCopies->value() );
   job->setWritingMode( m_writingModeWidget->writingMode() );
   job->setIgnoreReadErrors( m_checkIgnoreReadErrors->isChecked() );
   job->setReadRetries( m_spinRetries->value() );
+  job->setVerifyData( m_checkVerifyData->isChecked() );
   
-  hide();
+  if( !exitLoopOnHide() )
+    hide();
+
   dlg->startJob( job );
-  show();
+
   delete dlg;
+  delete job;
+
+  if( KConfigGroup( k3bcore->config(), "General Options" ).readBoolEntry( "keep action dialogs open", false ) &&
+      !exitLoopOnHide() )
+    show();
+  else
+    close();
 }
 
 
@@ -262,19 +291,18 @@ void K3bDvdCopyDialog::loadUserDefaults( KConfigBase* c )
 
   m_writerSelectionWidget->loadConfig( c );
 
-  m_tempDirSelectionWidget->setTempPath( K3b::defaultTempPath() );
-
   m_writingModeWidget->loadConfig( c );
 
-  m_tempDirSelectionWidget->setTempPath( K3b::defaultTempPath() );
+  m_tempDirSelectionWidget->readConfig( c );
 
   m_checkSimulate->setChecked( c->readBoolEntry( "simulate", false ) );
-  m_checkOnTheFly->setChecked( c->readBoolEntry( "on_the_fly", false ) );
+  m_checkCacheImage->setChecked( !c->readBoolEntry( "on_the_fly", false ) );
   m_checkOnlyCreateImage->setChecked( c->readBoolEntry( "only_create_image", false ) );
   m_checkDeleteImages->setChecked( c->readBoolEntry( "remove_image", true ) );
   m_checkIgnoreReadErrors->setChecked( c->readBoolEntry( "ignore read errors", false ) );
   m_spinRetries->setValue( c->readNumEntry( "retries", 128 ) );
   m_spinCopies->setValue( c->readNumEntry( "copies", 1 ) );
+  m_checkVerifyData->setChecked( c->readBoolEntry( "verify data", false ) );
 
   slotToggleAll();
 }
@@ -285,16 +313,18 @@ void K3bDvdCopyDialog::saveUserDefaults( KConfigBase* c )
   m_tempDirSelectionWidget->saveConfig();
 
   m_writingModeWidget->saveConfig( c );
+  m_tempDirSelectionWidget->saveConfig( c );
 
-  c->writeEntry( "source_device", m_comboSourceDevice->selectedDevice()->devicename() );
+  c->writeEntry( "source_device", m_comboSourceDevice->selectedDevice() ? m_comboSourceDevice->selectedDevice()->devicename() : QString() );
 
   c->writeEntry( "simulate", m_checkSimulate->isChecked() );
-  c->writeEntry( "on_the_fly", m_checkOnTheFly->isChecked() );
+  c->writeEntry( "on_the_fly", !m_checkCacheImage->isChecked() );
   c->writeEntry( "only_create_image", m_checkOnlyCreateImage->isChecked() );
   c->writeEntry( "remove_image", m_checkDeleteImages->isChecked() );
   c->writeEntry( "ignore read errors", m_checkIgnoreReadErrors->isChecked() );
   c->writeEntry( "retries", m_spinRetries->value() );
   c->writeEntry( "copies", m_spinCopies->value() );
+  c->writeEntry( "verify data", m_checkVerifyData->isChecked() );
 
   m_writerSelectionWidget->saveConfig( c );
 
@@ -306,72 +336,116 @@ void K3bDvdCopyDialog::saveUserDefaults( KConfigBase* c )
 void K3bDvdCopyDialog::loadK3bDefaults()
 {
   m_writerSelectionWidget->loadDefaults();
-  m_tempDirSelectionWidget->setTempPath( KGlobal::dirs()->resourceDirs( "tmp" ).first() );
+  m_tempDirSelectionWidget->setTempPath( K3b::defaultTempPath() );
 
   m_writingModeWidget->setWritingMode( K3b::WRITING_MODE_AUTO );
 
   m_checkSimulate->setChecked( false );
-  m_checkOnTheFly->setChecked( false );
+  m_checkCacheImage->setChecked( true );
   m_checkOnlyCreateImage->setChecked( false );
   m_checkDeleteImages->setChecked( true );
   m_checkIgnoreReadErrors->setChecked(false);
   m_spinCopies->setValue( 1 );
   m_spinRetries->setValue(128);
+  m_checkVerifyData->setChecked( false );
 
   slotToggleAll();
 }
 
 
-void K3bDvdCopyDialog::slotToggleAll()
+void K3bDvdCopyDialog::toggleAll()
 {
+  updateOverrideDevice();
+
   m_checkSimulate->setDisabled( m_checkOnlyCreateImage->isChecked() );
-  m_checkOnTheFly->setDisabled( m_checkOnlyCreateImage->isChecked() );
+  m_checkCacheImage->setDisabled( m_checkOnlyCreateImage->isChecked() );
 
   K3bDevice::Device* dev = m_writerSelectionWidget->writerDevice();
   if( dev ) {
+    // select the proper writing modes
+    // if writing and reading devices are the same we cannot use 
+    // K3bWritingModeWidget::determineSupportedModesFromMedium since the inserted medium is not the one we 
+    // will be using for burning. In that case we go the old fashioned way.
+    if( dev == m_comboSourceDevice->selectedDevice() ) {
+      int modes = 0;
+      if( dev->type() & (K3bDevice::DVDR|K3bDevice::DVDRW) ) {
+	modes |= K3b::DAO|K3b::WRITING_MODE_RES_OVWR;
+	if( dev->featureCurrent( K3bDevice::FEATURE_INCREMENTAL_STREAMING_WRITABLE ) != 0 )
+	  modes |= K3b::WRITING_MODE_INCR_SEQ;
+      }
 
-    if( (dev->type() & (K3bDevice::DVDPR|K3bDevice::DVDPRW)) &&
-	!(dev->type() & (K3bDevice::DVDR|K3bDevice::DVDRW)) ) {
-      // no simulation support for DVD+R(W) only drives
-      m_checkSimulate->setChecked(false);
-      m_checkSimulate->setEnabled(false);
-      
-      // what about the writing mode? Wy just say "overwrite" for DVD+R(W) for now
-      m_writingModeWidget->setSupportedModes( K3b::WRITING_MODE_RES_OVWR );
-    }
-    else {
-      // DVD-R(W) supported
-      m_writingModeWidget->setSupportedModes( K3b::WRITING_MODE_RES_OVWR|K3b::WRITING_MODE_INCR_SEQ|K3b::DAO );
+      m_writingModeWidget->setSupportedModes( modes );
       m_checkSimulate->setDisabled( m_checkOnlyCreateImage->isChecked() );
     }
-
-    if( m_comboSourceDevice->selectedDevice() == dev ) {
-      m_checkOnTheFly->setEnabled( false );
-      m_checkOnTheFly->setChecked( false );
-    }
     else {
-      m_checkOnTheFly->setDisabled( m_checkOnlyCreateImage->isChecked() );
+      m_writingModeWidget->determineSupportedModesFromMedium( dev );
+
+      if( k3bappcore->mediaCache()->diskInfo( dev ).mediaType() & K3bDevice::MEDIA_DVD_PLUS_ALL ) {
+	// no simulation support for DVD+R(W) media
+	m_checkSimulate->setChecked(false);
+	m_checkSimulate->setEnabled(false);
+      }
+      else {
+	m_checkSimulate->setDisabled( m_checkOnlyCreateImage->isChecked() );
+      }
     }
   }
 
+  
+
   m_writingModeWidget->setDisabled( m_checkOnlyCreateImage->isChecked() );
   m_writerSelectionWidget->setDisabled( m_checkOnlyCreateImage->isChecked() );
-  m_tempDirSelectionWidget->setDisabled( m_checkOnTheFly->isChecked() && !m_checkOnlyCreateImage->isChecked() );
+  m_tempDirSelectionWidget->setDisabled( !m_checkCacheImage->isChecked() && !m_checkOnlyCreateImage->isChecked() );
   m_writingModeWidget->setDisabled( m_checkOnlyCreateImage->isChecked() );
-  m_checkDeleteImages->setDisabled( m_checkOnlyCreateImage->isChecked() || m_checkOnTheFly->isChecked() );
+  m_checkDeleteImages->setDisabled( m_checkOnlyCreateImage->isChecked() || !m_checkCacheImage->isChecked() );
   m_spinCopies->setDisabled( m_checkSimulate->isChecked() || m_checkOnlyCreateImage->isChecked() );
-  if( m_checkOnlyCreateImage->isChecked() )
-    m_checkDeleteImages->setChecked( false );
+  m_checkVerifyData->setDisabled( m_checkOnlyCreateImage->isChecked() || m_checkSimulate->isChecked() );
   
-  m_buttonStart->setEnabled( m_comboSourceDevice->selectedDevice() && 
-			     (dev || m_checkOnlyCreateImage->isChecked()) );
+  setButtonEnabled( START_BUTTON, m_comboSourceDevice->selectedDevice() && 
+		    (dev || m_checkOnlyCreateImage->isChecked()) );
 }
 
 
-void K3bDvdCopyDialog::slotDeviceManagerChanged( K3bDevice::DeviceManager* dm )
+void K3bDvdCopyDialog::slotSourceMediumChanged( K3bDevice::Device* dev )
 {
-  m_comboSourceDevice->refreshDevices( dm->cdReader() );
+  updateOverrideDevice();
+
+  K3bMedium medium = k3bappcore->mediaCache()->medium( dev );
+
+  m_writerSelectionWidget->setWantedMediumType( k3bappcore->mediaCache()->diskInfo( dev ).numLayers() > 1 &&
+						k3bappcore->mediaCache()->diskInfo( dev ).size().mode1Bytes() > 4700372992LL
+						? K3bDevice::MEDIA_WRITABLE_DVD_DL
+						: K3bDevice::MEDIA_WRITABLE_DVD_SL );
+
+  m_tempDirSelectionWidget->setNeededSize( neededSize() );
+
+  m_tempDirSelectionWidget->setTempPath( m_tempDirSelectionWidget->tempDirectory() + medium.volumeId().lower() + ".iso" );
+
+  toggleAll();
 }
 
 
+void K3bDvdCopyDialog::updateOverrideDevice()
+{
+  if( !m_checkCacheImage->isChecked() )
+    m_writerSelectionWidget->setOverrideDevice( 0 );
+  else
+    m_writerSelectionWidget->setOverrideDevice( m_comboSourceDevice->selectedDevice(),
+						i18n("Use the same device for burning"),
+						i18n("<qt>Use the same device for burning <i>(Or insert another medium)</i>") );
+}
+
+
+KIO::filesize_t K3bDvdCopyDialog::neededSize() const
+{
+  K3bMedium medium = k3bappcore->mediaCache()->medium( m_comboSourceDevice->selectedDevice() );
+
+  if( medium.diskInfo().diskState() == K3bDevice::STATE_NO_MEDIA )
+    return 0;
+  else if( medium.diskInfo().mediaType() & (K3bDevice::MEDIA_DVD_RW_OVWR|K3bDevice::MEDIA_DVD_PLUS_RW) )
+    return (KIO::filesize_t)medium.iso9660Descriptor().volumeSpaceSize * (KIO::filesize_t)2048;
+  else
+    return medium.diskInfo().size().mode1Bytes();
+}
+
 #include "k3bdvdcopydialog.moc"
