@@ -4,7 +4,7 @@
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 class QCheckBox;
 class K3bWriterSelectionWidget;
 class QLabel;
-class K3bIso9660ImageWritingJob;
 class KURL;
 class K3bMd5Job;
 class K3bWritingModeWidget;
@@ -32,6 +31,9 @@ class K3bListView;
 class QSpinBox;
 class QDragEnterEvent;
 class QDropEvent;
+class KListView;
+class QListViewItem;
+class QPoint;
 
 
 /**
@@ -53,7 +55,7 @@ class K3bIsoImageWritingDialog : public K3bInteractionDialog
   void slotWriterChanged();
   void slotMd5JobPercent( int );
   void slotMd5JobFinished( bool );
-  void slotMd5SumCompare();
+  void slotContextMenu( KListView*, QListViewItem*, const QPoint& pos );
 
  protected:
   void loadUserDefaults( KConfigBase* );
