@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3baudiodatasourceviewitem.cpp 526655 2006-04-05 10:04:16Z trueg $
+ * $Id: k3baudiodatasourceviewitem.cpp 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2004 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,9 +36,9 @@ K3bAudioDataSourceViewItem::K3bAudioDataSourceViewItem( K3bAudioTrackViewItem* p
   f.setItalic( true );
   setFont( 3, f );
 
-  setMarginVertical( 2 );
+  //  setMarginVertical( 2 );
 
-  // greyed out filename
+  // gray out filename
   setForegroundColor( 5, listView()->palette().disabled().foreground() );
 
   // smaller filename
@@ -56,11 +56,11 @@ QString K3bAudioDataSourceViewItem::text( int i ) const
 {
   switch( i ) {
   case 3:
-    return " " + m_source->type() + " ";
+    return m_source->type();
   case 4:
     return m_source->length().toString();
   case 5:
-    return " " + m_source->sourceComment() + " ";
+    return m_source->sourceComment();
   default:
     return QString::null;
   }
@@ -69,11 +69,15 @@ QString K3bAudioDataSourceViewItem::text( int i ) const
 
 void K3bAudioDataSourceViewItem::setText( int col, const QString& text )
 {
+  //
+  // See K3bAudioTrackViewItem::setText for an explanation why we have to check if
+  // the value really changed
+  //
   if( col == 4 ) {
     if( K3bAudioZeroData* zero = dynamic_cast<K3bAudioZeroData*>( source() ) ) {
       bool ok;
       K3b::Msf f = K3b::Msf::fromString( text, &ok );
-      if( ok )
+      if( ok && f != zero->length() )
 	zero->setLength( f );
     }
   }
