@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3bmovixdvdburndialog.h 401319 2005-03-28 16:26:58Z trueg $
+ * $Id: k3bmovixdvdburndialog.h 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 class K3bMovixDvdDoc;
 class K3bMovixOptionsWidget;
 class K3bDataImageSettingsWidget;
-class K3bDataAdvancedImageSettingsWidget;
-class K3bDataVolumeDescWidget;
 class QCheckBox;
 
 
@@ -39,21 +37,18 @@ class K3bMovixDvdBurnDialog : public K3bProjectBurnDialog
  protected slots:
   void slotStartClicked();
 
-  void toggleAllOptions();
-
  protected:
   void saveSettings();
   void readSettings();
   void loadK3bDefaults();
   void loadUserDefaults( KConfigBase* );
   void saveUserDefaults( KConfigBase* );
+  void toggleAll();
 
  private:
   K3bMovixDvdDoc* m_doc;
   K3bMovixOptionsWidget* m_movixOptionsWidget;
-  K3bDataVolumeDescWidget* m_volumeDescWidget;
   K3bDataImageSettingsWidget* m_imageSettingsWidget;
-  K3bDataAdvancedImageSettingsWidget* m_advancedImageSettingsWidget;
 
   QCheckBox* m_checkVerify;
 };
