@@ -1,10 +1,10 @@
 /*
 *
-* $Id: k3bvideocdrippingdialog.cpp 401319 2005-03-28 16:26:58Z trueg $
+* $Id: k3bvideocdrippingdialog.cpp 640370 2007-03-07 19:44:32Z trueg $
 * Copyright (C) 2003 Christian Kvasny <chris@k3b.org>
 *
 * This file is part of the K3b project.
-* Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+* Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <kurlrequester.h>
 #include <kdebug.h>
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
 
 // qt includes
 #include <qgroupbox.h>
@@ -104,7 +105,7 @@ void K3bVideoCdRippingDialog::setupGui()
     groupDirectoryLayout->addWidget( necessarySizeBox, 2, 1 );
 
     // ---------------------------------------------------- Options group ---
-    QGroupBox* groupOptions = new QGroupBox( 4, Qt::Vertical, i18n( "Options" ), frame );
+    QGroupBox* groupOptions = new QGroupBox( 4, Qt::Vertical, i18n( "Settings" ), frame );
 
     m_ignoreExt = new QCheckBox( i18n( "Ignore /EXT/PSD_X.VCD" ), groupOptions );
 
@@ -155,6 +156,15 @@ void K3bVideoCdRippingDialog::slotStartClicked()
     QStringList filesExists;
     QDir d;
     d.setPath( m_editDirectory ->url() );
+    if( !d.exists() ) {
+      if( KMessageBox::warningYesNo( this, i18n("Image folder '%1' does not exist. Do you want K3b to create it?").arg( m_editDirectory->url() ) )
+	  == KMessageBox::Yes ) {
+	if( !KStandardDirs::makeDir( m_editDirectory->url() ) ) {
+	  KMessageBox::error( this, i18n("Failed to create folder '%1'.").arg( m_editDirectory->url() ) );
+	  return;
+	}
+      }
+    }
     const QFileInfoList* list = d.entryInfoList();
     QFileInfoListIterator it( *list );
     QFileInfo* fi;
@@ -168,7 +178,7 @@ void K3bVideoCdRippingDialog::slotStartClicked()
         if( KMessageBox::questionYesNoList( this,
                                 i18n("Continue although the folder is not empty?"),
                                 filesExists,
-                                i18n("Files Exist") ) == KMessageBox::No )
+                                i18n("Files Exist"),KStdGuiItem::cont(),KStdGuiItem::cancel() ) == KMessageBox::No )
         return;
 
     m_videooptions ->setVideoCdIgnoreExt( m_ignoreExt ->isChecked() );
